﻿using Aop.Api;
using Aop.Api.Domain;
using Aop.Api.Request;
using Aop.Api.Response;
using Aop.Api.Util;
using Common;
using Dapper;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using RedisHelp;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Web;
using TGood.Xian.LogManager;
using WxPayAPI.Models;
using wxWebAPI.business;
using wxWebAPI.Common;
using wxWebAPI.Models;

namespace WxPayAPI
{
    public class ChargeLogic
    {
        public static string RealDataAPI = ConfigurationManager.AppSettings["realdataAPI"];
        public static string ControlAPI = ConfigurationManager.AppSettings["controlAPI"];
        public static string ChargeAPI = ConfigurationManager.AppSettings["chargeAPI"];
        public const string DemoQRCode = "888888888888888";
        public const string TestQRCode = "666666666666666";
        public static string WXBody = ConfigurationManager.AppSettings["WXBody"];
        public static readonly string RedisConnectionString = ConfigurationManager.ConnectionStrings["RedisExchangeHosts"].ConnectionString;
        public static string SendSMSCompany = "小马充电";
        public static string TemplateCode = "SMS_189525768";
        public static string SendSMSApi = "http://172.16.7.125:8782/sms/send";
        #region 充电策略和服务费

        private List<StrategyInfo> GetChargeStrategy(string stationID)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            List<StrategyInfo> chargeStrategyList = null;

            var station = redis.HashGet<LCO_Station>(RedisKey.Station, stationID);
            if (station == null)
            {
                LogEvent.WebAPILoger.Error($"stationID={stationID}，没有获取到局站信息。");
                chargeStrategyList = new List<StrategyInfo>();
                var strategyInfo = new StrategyInfo();
                strategyInfo.StartTime = new STimeInfo(0, 0, 0, true);
                strategyInfo.EndTime = new STimeInfo(23, 59, 59, true);
                strategyInfo.Price = 10 * 1000;//默认一度电10元,协议单位是厘
                strategyInfo.Valid = true;
                chargeStrategyList.Add(strategyInfo);
            }
            else
            {
                //    station.TodayElectricFeeList = GetTodayElectricityFee(station.StationId, DateTime.Now);
                //    station.TodayServiceFeeList = GetTodayServiceFeelist(station.StationId, DateTime.Now);
                //    station.CurrentElectricFee = GetCurrentElectricityFee(station.TodayElectricFeeList);
                //    station.CurrentServiceFee = GetCurrentServiceFee(station.TodayServiceFeeList);
                chargeStrategyList = GetChargeStrategy(station.StationId, DateTime.Now);
            }
            return chargeStrategyList;
        }

        private double GetServicePrice(string stationID)
        {
            var serviceFee = 0d;
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var station = redis.HashGet<LCO_Station>(RedisKey.Station, stationID);
            if (station == null)
            {
                LogEvent.WebAPILoger.Error($"stationID={stationID}，没有获取到局站信息。");
            }
            else
            {
                serviceFee = GetCurrentServiceFee(GetTodayServiceFeelist(station.StationId, DateTime.Now));
            }

            return serviceFee;

        }

        #endregion

        #region 订单
        public IEnumerable<Order> GetHistoryOrder(string userToken, int minID)
        {
            IEnumerable<Order> orders = new List<Order>();
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);

            //处理第一次请求没有最后一条记录ID的情况
            if (minID <= 0)
            {
                minID = int.MaxValue;
            }
            var openid = GetOpenIDByUserToken(userToken);

            string wxOrdersql = "";
            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {


                string phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
                if (openid.Length == 11)
                {
                    phone = openid;
                }

                List<string> openidList = new List<string>();
                if (!string.IsNullOrEmpty(phone))
                {
                    string sql1 = $"SELECT openid FROM lco_wxcustomer WHERE phone = '{phone}'";
                    openidList = connection.Query<string>(sql1, connection).ToList();
                }


                if (openidList.Count == 0)
                {
                    // 判断是app的openid
                    if (openid.Length == 11)
                    {
                        wxOrdersql = $@"SELECT    A.id,A.userid, A.orderid,A.stationid,A.stationname,A.deviceCode,A.starttime,A.endtime,A.paymenttype,
                                              A.power,A.chargecost AS ElectricityFee,A.servicecost AS ServiceFee, A.orderduration, A.endreasondesc,A.startsoc,A.endsoc,b.totalfee as PrepaidAmount,A.cardid
                                    FROM histroy_order_cost A LEFT JOIN lco_wxorder B ON A.orderid = B.orderid  
                                                WHERE  A.userid  = '{openid}' and  A .id < {minID}   ORDER BY  A.endtime desc ";
                    }
                    //兼顾微信刷卡启动、微信启动
                    else
                    {
                        wxOrdersql = $@"SELECT  A.id,A.userid, A.orderid,A.stationid,A.stationname,A.deviceCode,A.starttime,A.endtime,A.paymenttype,
                                              A.power,A.chargecost AS ElectricityFee,A.servicecost AS ServiceFee, A.orderduration, A.endreasondesc,A.startsoc,A.endsoc,b.totalfee as PrepaidAmount,A.cardid
                                    FROM histroy_order_cost A LEFT JOIN lco_wxorder B ON A.orderid = B.orderid  
                                                WHERE  A.userid like '{openid.Substring(0, 23)}%' and  A.id < {minID}  ORDER BY  A.endtime desc  ";
                    }
                }
                //当那个数据库lco_wxcustomer查询不到相关
                else
                {
                    wxOrdersql = $@"SELECT distinct TOP 20 * FROM(SELECT  A.id,A.userid, A.orderid,A.stationid,A.stationname,A.deviceCode,A.starttime,A.endtime,A.paymenttype,
                                                       A.power,A.chargecost AS ElectricityFee,A.servicecost AS ServiceFee, A.orderduration, A.endreasondesc,A.startsoc,A.endsoc,b.totalfee as PrepaidAmount,A.cardid
                                                FROM histroy_order_cost A LEFT JOIN lco_wxorder B ON A.orderid = B.orderid
                                                WHERE substring(A.userid,0,23) in ( SELECT substring(openid,0,23) FROM lco_wxcustomer WHERE phone ='{phone}') AND A.id < 9999999 
                                union               
                                               SELECT   A.id,A.userid, A.orderid,A.stationid,A.stationname,A.deviceCode,A.starttime,A.endtime,A.paymenttype,
                                               A.power,A.chargecost AS ElectricityFee,A.servicecost AS ServiceFee, A.orderduration, A.endreasondesc,A.startsoc,A.endsoc,b.totalfee as PrepaidAmount,A.cardid
        FROM histroy_order_cost A LEFT JOIN lco_wxorder B ON A.orderid = B.orderid
        WHERE A.userid = '{openid}' AND A.id < {minID}) C WHERE C.id<{minID} ORDER BY C.endtime desc";
                }
                orders = SqlMapper.Query<Order>(connection, wxOrdersql);

            }
            return orders;
        }

        public IEnumerable<Order> GetHistoryOrder(string userToken, int year, int month)
        {
            var openid = GetOpenIDByUserToken(userToken);

            DateTime starttime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            try
            {
                starttime = new DateTime(year, month, 1);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetHistoryOrder日期参数不对，year={year},month={month}", ex);
            }
            DateTime endtime = starttime.AddMonths(1);

            IEnumerable<Order> orders = new List<Order>();
            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var sql = $@"SELECT A.id,A.userid,A.orderid,A.stationid,A.stationname,A.starttime,A.endtime,A.power,
                        A.chargecost AS ElectricityFee,A.servicecost AS ServiceFee, A.orderduration, A.endreasondesc  
                        FROM histroy_order_cost A
                        WHERE A.userid = '{openid}' AND endtime>='{starttime.ToString("yyyy-MM-dd")}' AND endtime<'{endtime.ToString("yyyy-MM-dd")}' ORDER BY id DESC ";
                orders = SqlMapper.Query<Order>(connection, sql);
            }
            return orders;
        }
        public IEnumerable<Order> GetHistoryOrder(string userid)
        {
            IEnumerable<Order> orders = new List<Order>();
            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var sql = $@"SELECT A.id,A.userid,A.orderid,A.stationid,A.stationname,A.starttime,A.endtime,A.power,
                        A.chargecost AS ElectricityFee,A.servicecost AS ServiceFee, A.orderduration, A.endreasondesc  
                        FROM histroy_order_cost A
                        WHERE A.userid = '{userid}'  ORDER BY id DESC ";
                orders = SqlMapper.Query<Order>(connection, sql);
            }
            //补全历史订单中没有的信息
            foreach (var item in orders)
            {
                //item.userName = wxDeviceInfo.nickName;
                item.power = Math.Round(item.power, 2);
                item.ElectricityFee = Math.Round(item.ElectricityFee, 2);
                item.ServiceFee = Math.Round(item.ServiceFee, 2);
            }

            return orders;
        }

        public IEnumerable<dynamic> GetMonthOrder(string userToken, string month = null, string map = null)
        {
            IEnumerable<dynamic> orders = new List<dynamic>();
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            string openid = string.Empty;
            try
            {
                openid = GetOpenIDByUserToken(userToken);


                using (IDbConnection connection = WechatUtility.CreateDBConnection())
                {
                    if (month == null)
                    {
                        var sql = $@"SELECT  sum(power) AS totalPower ,sum(chargecost) AS totalChargeCost,month(starttime) AS month 
                           FROM   histroy_order_cost 
                           WHERE userid = '{openid}' and DATEDIFF(month,starttime,getdate())<6 
                           GROUP BY userid,month(starttime)";
                        orders = SqlMapper.Query<dynamic>(connection, sql);
                    }
                    else if (month != null && map == null)
                    {
                        string sql = $@"SELECT * FROM  (SELECT count(id) as TimesByDay, sum(power) AS power,sum(chargecost) AS chargecost,sum(orderduration) AS duration, day(starttime) AS day FROM histroy_order_cost WHERE userid = '{openid}' AND month(starttime) = {month}   GROUP BY day(starttime)) AS a ORDER BY a.day";
                        orders = SqlMapper.Query<sumDuration>(connection, sql);
                    }
                    else
                    {
                        List<MonthOrder> monthorder = new List<MonthOrder>();
                        string sql = $"SELECT stationid,stationname, count(*) AS num ,sum(power) AS TotalPower,sum(chargecost) AS TotalChargeCost FROM histroy_order_cost WHERE userid='{openid}' AND month(starttime) = {month} GROUP BY stationname,stationid";
                        monthorder = SqlMapper.Query<MonthOrder>(connection, sql).ToList();
                        if (orders != null)
                        {
                            foreach (var item in monthorder)
                            {
                                item.Longitude = redis.HashGet<LCO_Station>(RedisKey.Station, item.stationid).Longitude;
                                item.Latitude = redis.HashGet<LCO_Station>(RedisKey.Station, item.stationid).Latitude;
                            }
                        }
                        return monthorder;
                    }
                }
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"获取月账单异常usertoken={userToken},openid={openid},month={month}", e);
                throw e;

            }

            return orders;
        }

        public List<Order> GetActiveOrderByOpenId(string openId)
        {
            List<Order> orders = new List<Order>();
            List<string> chargingDevices = GetChargingDevices(openId);

            if (chargingDevices == null)
            {
                return orders;
            }

            //取每个设备的activeOrder
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.RealData, RedisConnectionString);
            foreach (var item in chargingDevices)
            {
                Order activeOrder = redis.HashGet<Order>(RedisKey.ActiveOrder, item);
                if (activeOrder != null && string.IsNullOrEmpty(activeOrder.endtime.ToString()))
                {
                    orders.Add(activeOrder);
                }
            }

            //补全历史订单中没有的信息
            if (orders != null)
            {
                foreach (var item in orders)
                {
                    //item.userName = wxDeviceInfo.nickName;
                    item.power = Math.Round(item.power, 2);
                    item.ElectricityFee = Math.Round(item.ElectricityFee, 2);
                    item.ServiceFee = Math.Round(item.ServiceFee, 2);
                }
            }

            return orders;
        }

        public JObject GetActiveOrderByDeviceCode(string deviceCode)
        {
            JObject result = new JObject();
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.RealData, RedisConnectionString);
            Order activeOrder = redis.HashGet<Order>(RedisKey.ActiveOrder, deviceCode);

            if (activeOrder == null)
            {
                result.Add("code", 0);
                result.Add("data", "订单不存在");
                return result;
            }

            if (!string.IsNullOrEmpty(activeOrder.endtime.ToString()))
            {
                result.Add("code", 0);
                result.Add("data", "订单已结束");
                return result;
            }

            //补全订单中没有的信息
            activeOrder.power = Math.Round(activeOrder.power, 2);
            activeOrder.ElectricityFee = Math.Round(activeOrder.ElectricityFee, 2);
            activeOrder.ServiceFee = Math.Round(activeOrder.ServiceFee, 2);
            result.Add("code", 1);
            result.Add("data", JProperty.FromObject(activeOrder));
            return result;
        }

        public Order GetCurrentOrder(string userToken, string deviceCode)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            //var openid = GetOpenIDByUserToken(userToken);
            System.Threading.Thread.Sleep(2 * 1000);
            Order order = new Order();
            List<WXDeviceInfo> wxDeviceInfo = NewGetDeviceInfoByUserToken(userToken);
            if (wxDeviceInfo != null)
            {
                foreach (var item in wxDeviceInfo)
                {
                    if (item.DeviceCode == deviceCode)
                    {
                        order = GetOrderByOrderID(item);
                        break;
                    }
                }
                //更新chargeingDevice表

                List<string> newList = new List<string>();
                List<string> newList1 = new List<string>();
                var openid = redis.HashGetString(RedisKey.Token, userToken);
                var chargeList = redis.HashGet<List<string>>(RedisKey.ChargingDevice, openid);
                if (chargeList != null)
                {

                    chargeList.Remove(deviceCode);

                    foreach (var newdeviceCode in chargeList)
                    {
                        var deviceinfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, newdeviceCode);
                        if (deviceinfo.Openid == openid)
                        {
                            newList1.Add(newdeviceCode);
                        }
                    }

                    if (newList1 != null && newList1.Any())
                    {
                        redis.HashSet<List<string>>(RedisKey.ChargingDevice, openid, newList1);
                    }
                    else
                    {
                        redis.HashDelete(RedisKey.ChargingDevice, openid);
                    }

                }
            }
            return order;
        }

        public Order GetCurrentOrder(string userToken)
        {
            //var openid = GetOpenIDByUserToken(userToken);
            System.Threading.Thread.Sleep(2 * 1000);
            var wxDeviceInfo = GetDeviceInfoByUserToken(userToken);
            var order = GetOrderByOrderID(wxDeviceInfo);

            return order;
        }

        private Order GetOrderByOrderID(WXDeviceInfo wxDeviceInfo)
        {
            Order order = null;

            #region 体验扫码启动Demo
            if (wxDeviceInfo != null && wxDeviceInfo.DeviceCode.Length >= 15 && wxDeviceInfo.DeviceCode.Substring(0, 15).Equals(DemoQRCode))
            {
                order = new Order();
                order.orderid = wxDeviceInfo.OrderID;
                order.ServiceFee = 8;
                order.ElectricityFee = 8;
                order.endtime = DateTime.Now;
                order.power = 8;
                order.PrepaidAmount = (decimal)(wxDeviceInfo.Total_fee / 100.0);
                order.starttime = DateTime.Parse(wxDeviceInfo.StartTime);
                order.stationid = wxDeviceInfo.StationCode;
                order.stationname = "测试站点";
                order.userid = wxDeviceInfo.Openid;
                order.userName = "测试账户";
                return order;
            }
            #endregion

            if (wxDeviceInfo != null)
            {
                // 获取设备的开始订单，为了得到原始订单号
                RedisHelper redisRealData = new RedisHelper((int)EnumUserRedisNum.RealData, RedisConnectionString);
                active_order deviceActiveOrder = redisRealData.HashGet<active_order>(RedisKey.ActiveOrder, wxDeviceInfo.DeviceCode);

                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                #region 优先从前置机获取当前订单

                RedisHelper redisEndOrder = new RedisHelper((int)EnumUserRedisNum.EndOrder, RedisConnectionString);
                for (int i = 1; i <= 3; i++)
                {
                    var endOrderInfo = redisEndOrder.HashGet<EndOrderInfo>(RedisKey.HisOrder, wxDeviceInfo.OrderID);
                    if (endOrderInfo != null)
                    {
                        order = new Order();
                        order.orderid = endOrderInfo.OrderID;
                        order.ElectricityFee = Math.Round(endOrderInfo.ChargeCost.HasValue ? endOrderInfo.ChargeCost.Value : 0, 2);
                        order.endtime = endOrderInfo.EndTime;
                        order.power = Math.Round(endOrderInfo.Power, 2);
                        order.PrepaidAmount = (decimal)(wxDeviceInfo.Total_fee / 100.0);
                        order.ServiceFee = Math.Round(endOrderInfo.ServiceCost.HasValue ? endOrderInfo.ServiceCost.Value : 0, 2);
                        order.starttime = endOrderInfo.StartTime.HasValue ? endOrderInfo.StartTime.Value : DateTime.Parse(wxDeviceInfo.StartTime);
                        order.stationid = endOrderInfo.StationId;
                        order.stationname = endOrderInfo.StationName;
                        order.userid = endOrderInfo.UserID;
                        order.userName = wxDeviceInfo.NickName;
                        order.OriginalOrderID = endOrderInfo.Guid;
                        //有时用户会多次获取当前订单，这个集合wxpay专门有个线程删除前一天的
                        //redis.HashDelete(RedisKey.HisOrder, wxDeviceInfo.OrderID);
                        break;
                    }
                    System.Threading.Thread.Sleep(i * 1000);
                }

                #endregion

                #region 如果前置机订单中找不到，则在Redis中计算订单
                if (order == null)
                {
                    if (wxDeviceInfo.OrderState != EnumWXOrderState.充电结束 && wxDeviceInfo.OrderState != EnumWXOrderState.退费完成)
                    {
                        System.Threading.Thread.Sleep(3 * 1000);

                        wxDeviceInfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, wxDeviceInfo.DeviceCode);
                    }
                    PayLogic paylogic = new PayLogic();
                    List<Price> servicefee = paylogic.GetNewServiceFee(wxDeviceInfo.StationID);

                    order = new Order();
                    order.orderid = wxDeviceInfo.OrderID;
                    order.ServiceFee = (decimal)Math.Ceiling(GetCurrentServiceFee(servicefee) * wxDeviceInfo.PowerConsumption * 100) / 100;
                    //单位分别是分、厘、元
                    order.ElectricityFee = (decimal)Math.Ceiling((wxDeviceInfo.Total_fee * 10 - wxDeviceInfo.RemainingFee) / 10.0) / 100 - order.ServiceFee;
                    order.endtime = DateTime.Now;
                    order.power = (decimal)Math.Round(wxDeviceInfo.PowerConsumption, 2);
                    order.PrepaidAmount = (decimal)(wxDeviceInfo.Total_fee / 100.0);
                    order.starttime = DateTime.Parse(wxDeviceInfo.StartTime);
                    order.stationid = wxDeviceInfo.StationCode;
                    order.stationname = wxDeviceInfo.StationName;
                    order.userid = wxDeviceInfo.Openid;
                    order.userName = wxDeviceInfo.NickName;
                    order.OriginalOrderID = deviceActiveOrder == null ? wxDeviceInfo.OrderID : deviceActiveOrder.orderid;

                    #endregion
                }
            }
            return order;
        }

        private bool IsSystemInterface(string JKCode)
        {
            string jkProtocol = GetJKProtocol(JKCode);
            return jkProtocol.ToLower() == "systeminterface";
        }

        private bool Is104Protocol(string JKCode)
        {
            //判断是否104
            string jkProtocol = GetJKProtocol(JKCode);
            return jkProtocol.ToLower() == "centralizedcontrol";
        }

        private string GetJKProtocol(string JKCode)
        {
            RedisHelper redisStruct = new RedisHelper((int)EnumUserRedisNum.Struct, RedisConnectionString);
            string JKProtocol = redisStruct.HashGetString(RedisKey.StationProtocol, JKCode);

            return JKProtocol;
        }

        public Order GetDeviceLastOrder(string deviceCode)
        {
            IEnumerable<Order> orders = new List<Order>();

            var sql = $@"SELECT TOP 1 A.userid,A.orderid,A.stationid,A.stationname,A.deviceCode,A.starttime,A.endtime,
                        A.power,A.chargecost AS ElectricityFee,A.servicecost AS ServiceFee, A.orderduration, A.endreasondesc,A.startsoc,A.endsoc,
                        A.guid AS OriginalOrderID
                        FROM histroy_order_cost A
                        WHERE A.devicecode = @devicecode AND A.chargeway!=-1 ORDER BY endtime DESC ";
            var param = new DynamicParameters();
            param.Add("@devicecode", deviceCode);
            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                orders = SqlMapper.Query<Order>(connection, sql, param);
            }
            return (orders != null && orders.Any()) ? orders.First() : null;
        }


        #endregion

        #region 充电实时数据
        /// <summary>
        /// 获取充电实时数据
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        public ChargeData GetRealData(string userToken, string deviceCode)
        {
            //从userToken查询在哪一个设备上充电
            List<WXDeviceInfo> wxDeviceInfo = NewGetDeviceInfoByUserToken(userToken);
            ChargeData result = null;
            if (wxDeviceInfo != null && wxDeviceInfo.Any())
            {
                foreach (var item in wxDeviceInfo)
                {
                    if (item != null && item.OrderState != EnumWXOrderState.未开始 && item.DeviceCode == deviceCode)
                    {
                        result = ConvertWXDeviceInfoToChargeData(item);
                        break;
                    }
                }
            }

            return result;

        }

        public ChargeData GetRealData(string userToken)
        {
            //从userToken查询在哪一个设备上充电
            WXDeviceInfo wxDeviceInfo = GetDeviceInfoByUserToken(userToken);
            ChargeData result = null;

            if (wxDeviceInfo != null && wxDeviceInfo.OrderState != EnumWXOrderState.未开始)
            {
                result = ConvertWXDeviceInfoToChargeData(wxDeviceInfo);
            }
            else
            {
                //遇到付款通知稍慢的情况，等3秒钟再试一次
                System.Threading.Thread.Sleep(3 * 1000);
                wxDeviceInfo = GetDeviceInfoByUserToken(userToken);
                result = ConvertWXDeviceInfoToChargeData(wxDeviceInfo);
            }
            return result;

        }

        //public JObject GetRealData(string userToken, string deviceCode)
        //{
        //    JObject result = new JObject();
        //    List<string> chargingDevice = GetChargingDevicesByUserToken(userToken);
        //    if (chargingDevice == null || !chargingDevice.Contains(deviceCode))
        //    {
        //        result.Add("code", 0);
        //        result.Add("data", "非本人使用设备");
        //        return result;
        //    }
        //    WXDeviceInfo wxDeviceInfo = GetDeviceInfo(deviceCode);
        //    if (wxDeviceInfo != null && wxDeviceInfo.OrderState != EnumWXOrderState.未开始)
        //    {
        //        result.Add("code", 1);
        //        result.Add("data", JProperty.FromObject(ConvertWXDeviceInfoToChargeData(wxDeviceInfo)));
        //    }
        //    else
        //    {
        //        //遇到付款通知稍慢的情况，等3秒钟再试一次
        //        System.Threading.Thread.Sleep(3 * 1000);
        //        wxDeviceInfo = GetDeviceInfo(deviceCode);
        //        if (wxDeviceInfo != null)
        //        {
        //            result.Add("code", 1);
        //            result.Add("data", JProperty.FromObject(ConvertWXDeviceInfoToChargeData(wxDeviceInfo)));
        //        }
        //        else
        //        {
        //            result.Add("code", 0);
        //            result.Add("data", "未获取到实时数据");
        //        }
        //    }

        //    return result;

        //}

        private ChargeData ConvertWXDeviceInfoToChargeData(WXDeviceInfo deviceInfo)
        {
            int status = GetGunStatus(deviceInfo.DeviceCode);
            //市政府院内充电站设备无连接状态
            if (status == (int)EnumGunStatus.未插枪 && deviceInfo.StationID == "6101120047")
            {
                status = (int)EnumGunStatus.连接;
            }

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.BMS, RedisConnectionString);
            var result = new ChargeData();
            if (deviceInfo != null)
            {
                //转换Redis中的数据到小程序的充电数据
                result.Current = deviceInfo.Current;
                result.CurrentA = deviceInfo.CurrentA;
                result.CurrentB = deviceInfo.CurrentB;
                result.CurrentC = deviceInfo.CurrentC;
                result.FeeOfUse = Double.Parse(((deviceInfo.Total_fee * 10 - deviceInfo.RemainingFee) / 1000.0).ToString("F1"));
                result.DeviceType = (int)deviceInfo.DeviceType;
                // result.Duration = (string.IsNullOrEmpty(deviceInfo.Duration) || !deviceInfo.Duration.Contains("分钟")) ? "" : deviceInfo.Duration.Split(new string[] { "分钟" }, StringSplitOptions.None)[0] + "分钟";
                result.Power = deviceInfo.Power;
                result.Duration = deviceInfo.Duration;
                //result.RemainingTime = deviceInfo.RemainingTime;
                if (deviceInfo.ServiceFeeList != null && deviceInfo.ElectricityFeeList != null)
                {
                    result.ServiceFee = GetCurrentServiceFee(deviceInfo.ServiceFeeList).ToString("F4");
                    result.CurrentElectricityFee = GetCurrentElectricityFee(deviceInfo.ElectricityFeeList).ToString("F4");
                }
                else
                {
                    result.ServiceFee = deviceInfo.ServiceFee.ToString("F4");
                    result.CurrentElectricityFee = (GetCurrentElectricityFee(deviceInfo.ChargeStrategy) / 1000.0 - deviceInfo.ServiceFee).ToString("F4");
                }
                result.SOC = deviceInfo.SOC;
                result.StartTime = deviceInfo.StartTime;
                result.StationID = deviceInfo.StationCode;
                result.StationName = deviceInfo.StationName;
                result.Voltage = deviceInfo.Voltage;
                result.VoltageA = deviceInfo.VoltageA;
                result.VoltageB = deviceInfo.VoltageB;
                result.VoltageC = deviceInfo.VoltageC;
                result.PowerConsumption = string.IsNullOrEmpty(deviceInfo.Voltage) ? "" : deviceInfo.PowerConsumption.ToString("F4");
                result.Total_fee = Math.Round(deviceInfo.Total_fee / 100.0, 2);
                result.LastUpdateTime = deviceInfo.LastUpdateTime.ToString("MM-dd HH:mm");
                result.GunStatus = (EnumGunStatus)status;
                //result.GunStatus = deviceInfo.GunState;
                result.GunStatusName = Enum.GetName(typeof(EnumGunStatus), deviceInfo.GunState);
                result.OrderState = deviceInfo.OrderState;
                result.RemainingTime = calculateRemainTime(deviceInfo.DeviceCode);
                Gunsignal data = redis.HashGet<Gunsignal>(RedisKey.ChargeProcessData, deviceInfo.DeviceCode);
                if (data != null)
                {
                    result.gunsignal = data;
                }
            }
            return result;

        }

        internal GunInfo GetRealDataByDeviceAndSignal(string deviceCode, string[] signalIds = null)
        {
            var result = new GunInfo();

            //indexType=2 deviceCode; =1 deviceID
            var uri = ChargeAPI + $"GetGunInfo?index={deviceCode}&indexType=2";
            //var param = new StringBuilder();
            //foreach (var item in signalIds)
            //{
            //    param.Append($"&signalList={item}");
            //}
            //uri = uri + param.ToString();

            var apiResult = "";
            try
            {
                using (var httpClient = new HttpClient())
                {
                    apiResult = httpClient.GetStringAsync(uri).Result;
                }

                var jsonResult = JObject.Parse(apiResult);
                if (jsonResult.GetValue("code").ToString() == ((int)EnumAPIResult.Succeed).ToString())
                {
                    result = JsonConvert.DeserializeObject<GunInfo>(jsonResult.GetValue("result").ToString(Newtonsoft.Json.Formatting.None));
                }
                else
                {
                    LogEvent.WebAPILoger.Error($"GetRealDataByDeviceAndSignal Error = {jsonResult.GetValue("error").ToString(Newtonsoft.Json.Formatting.None)}，uri={uri}。");
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetRealDataByDeviceAndSignal Exception ，uri={uri}。", ex);
            }

            return result;
        }

        public int GetCurrentElectricityFee(List<StrategyInfo> chargeStrategy)
        {
            if (chargeStrategy == null || !chargeStrategy.Any())
            {
                return int.MaxValue;
            }

            DateTime now = DateTime.Now;
            int nowHour = now.Hour;
            int nowMinute = now.Minute;
            int nowSecond = now.Second;
            var currentStrategy = chargeStrategy.FirstOrDefault(p =>
            {
                if (p.Valid && p.StartTime.Valid && p.EndTime.Valid)
                {
                    var startime = new DateTime(now.Year, now.Month, now.Day, p.StartTime.Hour, p.StartTime.Minute, p.StartTime.Second);
                    var endtime = new DateTime(now.Year, now.Month, now.Day, p.EndTime.Hour, p.EndTime.Minute, p.EndTime.Second);
                    return now >= startime && now < endtime;
                }
                else
                {
                    return false;
                }
                //return (p.Valid)
                //    && (p.StartTime.Valid && p.StartTime.Hour <= nowHour && p.EndTime.Valid && p.EndTime.Hour >= nowHour)
                //    && (p.StartTime.Valid && p.StartTime.Minute <= nowMinute && p.EndTime.Valid && p.EndTime.Minute >= nowMinute)
                //    && (p.StartTime.Valid && p.StartTime.Second <= nowSecond && p.EndTime.Valid && p.EndTime.Second >= nowSecond);
            });
            int currentPrice = currentStrategy == null ? 0 : currentStrategy.Price;
            return currentPrice;
        }
        public List<Price> GetTodayServiceFeelist(string stationID, DateTime date)
        {
            List<Price> serviceFeeStrategyList = null;
            PriceStrategy nowServiceFeeStrategy = null;
            List<PriceStrategy> serviceFeeStrategy = null;

            if (!string.IsNullOrEmpty(stationID))
            {
                var now = DateTime.Now;
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WebAPI, RedisConnectionString);
                serviceFeeStrategy = redis.HashGet<List<PriceStrategy>>(RedisKey.NewServiceFee, stationID);
                if (serviceFeeStrategy != null && serviceFeeStrategy.Any())
                {
                    nowServiceFeeStrategy = serviceFeeStrategy.FirstOrDefault(p => p.UseTime <= now && p.EndUseTime > now);
                }
            }

            if (nowServiceFeeStrategy == null)
            {
                var serviceFeeStrategyToString = serviceFeeStrategy == null ? "null" : JsonConvert.SerializeObject(serviceFeeStrategy);
                LogEvent.CMUServerLoger.Error($"stationID={stationID}，没有获取到有效的服务费策略，策略={serviceFeeStrategyToString}。");

                serviceFeeStrategyList = new List<Price>();
                //var price = new Price();
                //price.startTime = new TimeSpan(0, 0, 0);
                //price.endTime = new TimeSpan(23, 59, 59);
                //price.price = 10;//默认一度电10元
                //serviceFeeStrategyList.Add(price);
            }
            else
            {
                nowServiceFeeStrategy.Strategy.ForEach(p => p.price = Double.Parse(p.price.ToString("F4")));
                serviceFeeStrategyList = nowServiceFeeStrategy.Strategy;
            }

            return serviceFeeStrategyList;
        }
        public List<Price> GetTodayElectricityFee(string stationID, DateTime date)
        {
            List<Price> electricityFeeStrategyList = null;
            PriceStrategy nowElectricityFeeStrategy = null;
            List<PriceStrategy> electricityFeeStrategy = null;

            if (!string.IsNullOrEmpty(stationID))
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WebAPI, RedisConnectionString);
                electricityFeeStrategy = redis.HashGet<List<PriceStrategy>>(RedisKey.NewElectricFee, stationID);
                if (electricityFeeStrategy != null && electricityFeeStrategy.Any())
                {
                    nowElectricityFeeStrategy = electricityFeeStrategy.FirstOrDefault(p => p.UseTime <= date && p.EndUseTime > date);
                }
            }

            if (nowElectricityFeeStrategy == null)
            {
                var electricityFeeStrategyToString = electricityFeeStrategy == null ? "null" : JsonConvert.SerializeObject(electricityFeeStrategy);
                LogEvent.CMUServerLoger.Error($"stationID={stationID}，没有获取到有效的电费策略，策略={electricityFeeStrategyToString}。");

                electricityFeeStrategyList = new List<Price>();
                //var price = new Price();
                //price.startTime = new TimeSpan(0, 0, 0);
                //price.endTime = new TimeSpan(23, 59, 59);
                //price.price = 10;//默认一度电10元
                //electricityFeeStrategyList.Add(price);
            }
            else
            {
                nowElectricityFeeStrategy.Strategy.ForEach(p => p.price = Double.Parse(p.price.ToString("F4")));
                electricityFeeStrategyList = nowElectricityFeeStrategy.Strategy;
            }

            return electricityFeeStrategyList;



        }
        private List<Price> GetTodayElectricityFee(List<List<Price>> electricFeeList)
        {
            if ( !electricFeeList.Any())
            {
                return null;
            }
            var historyElectricityFee = electricFeeList.Where(p => p.First().useTime <= DateTime.Now).ToList();
            if ( !historyElectricityFee.Any())
            {
                return null;
            }
            var todayElectricityFee = historyElectricityFee.OrderByDescending(p => p.First().useTime).First();
            return todayElectricityFee;
        }

        public double GetCurrentElectricityFee(List<Price> todayElectricityFee)
        {
            if (todayElectricityFee == null || !todayElectricityFee.Any())
            {
                return 0;
            }
            var currentElectricityFee = todayElectricityFee.FirstOrDefault(p => (p.startTime <= DateTime.Now.TimeOfDay && p.endTime >= DateTime.Now.TimeOfDay));
            return currentElectricityFee == null ? 0 : currentElectricityFee.price;
        }

        public double GetCurrentServiceFee(List<Price> todayServiceFee)
        {
            if (todayServiceFee == null || !todayServiceFee.Any())
            {
                return 0;
            }
            var currentElectricityFee = todayServiceFee.FirstOrDefault(p => (p.startTime <= DateTime.Now.TimeOfDay && p.endTime >= DateTime.Now.TimeOfDay));
            return currentElectricityFee == null ? 0 : currentElectricityFee.price;
        }

        private double GetTodayServiceFee(List<ServicePrice> serviceFeeList)
        {
            if (serviceFeeList == null || !serviceFeeList.Any())
            {
                return int.MaxValue;
            }
            var historyServiceFee = serviceFeeList.Where(p => p.startTime <= DateTime.Now).ToList();
            if (historyServiceFee == null || !historyServiceFee.Any())
            {
                return int.MaxValue;
            }
            return historyServiceFee.OrderByDescending(p => p.startTime).First().price;
        }

        private List<StrategyInfo> CreateChargeStrategy(List<Price> electricityFee, double serviceFee)
        {
            return CreateDiscountChargeStrategy(electricityFee, serviceFee, 1, 1);
        }

        public List<StrategyInfo> GetChargeStrategy(string stationID, DateTime date)
        {
            List<StrategyInfo> chargeStrategy = new List<StrategyInfo>();
            //ChargeStrategy chargeStartegy = new ChargeStrategy(); 
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Struct, RedisConnectionString);
            try
            {
                var chargeStartegy = redis.HashGet<List<ChargeStrategy>>(RedisKey.NewChargeStrategy, stationID);
                if (chargeStartegy != null)
                {
                    foreach (var item in chargeStartegy)
                    {
                        if (item.UseTime <= date && item.EndUseTime >= date)
                        {
                            chargeStrategy = item.Strategy;
                            return chargeStrategy;
                        }
                    }
                }
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"无法获取{stationID}场站对应策略", e);
                throw;
            }
            return new List<StrategyInfo>();
        }

        private List<StrategyInfo> CreateDiscountChargeStrategy(List<Price> electricityFee, double serviceFee,
            double electricityDiscount, double serviceDiscount)
        {
            List<StrategyInfo> chargeStrategy = new List<StrategyInfo>();
            if (electricityFee == null || !electricityFee.Any())
            {
                return null;
            }
            foreach (var item in electricityFee)
            {
                StrategyInfo strategy = new StrategyInfo();
                STimeInfo start = new STimeInfo(item.startTime.Hours, item.startTime.Minutes, item.startTime.Seconds, true);
                STimeInfo end = new STimeInfo(item.endTime.Hours, item.endTime.Minutes, item.endTime.Seconds, true);
                strategy.StartTime = start;
                strategy.EndTime = end;
                strategy.Price = (int)((item.price * electricityDiscount + serviceFee * serviceDiscount) * 1000);
                strategy.Valid = true;
                chargeStrategy.Add(strategy);
            }
            return chargeStrategy;
        }
        #endregion

        #region 启动停止充电
        private bool StopCharge(ControlRequest controlRequest)
        {
            ////根据userToken构造ControlRequest
            //var controlRequest = new ControlRequest();
            //controlRequest.ControlID = 5003;
            //controlRequest.ControlType = EnumControlType.Value;
            //controlRequest.DeviceCode = wxDeviceInfo.deviceCode;
            //controlRequest.DoubleValue = 0;
            //controlRequest.StringValue = "";
            //controlRequest.UserID = wxDeviceInfo.openid;
            //controlRequest.UserName = wxDeviceInfo.nickName;
            //controlRequest.ValidTime = DateTime.Now.AddHours(1);
            //var stopResult = StopCharge(controlRequest);

            var uri = ControlAPI + "SetControl";
            LogEvent.WebAPILoger.Debug("StopCharge, 停止充电指令");
            var result = SendControl(controlRequest, uri);

            return result;
        }

        private bool StopCharge(ChargeApplyInfo apply)
        {
            var uri = ControlAPI + "StartCharge";
            LogEvent.WebAPILoger.Info("StopCharge 停止充电指令");
            var result = SendControl(apply, uri);

            return result;
        }

        private bool SendControl(object postData, string uri)
        {
            bool result = false;

            var apiResult = "";
            using (var httpClient = new HttpClient())
            {
                string jsonString = JsonConvert.SerializeObject(postData);
                LogEvent.WebAPILoger.Debug("SendControl jsonString = " + jsonString);
                byte[] bytes = System.Text.Encoding.UTF8.GetBytes(jsonString);
                using (StreamContent sc = new StreamContent(new MemoryStream(bytes)))
                {
                    sc.Headers.ContentLength = bytes.Length;
                    sc.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                    var httpResponseMessage = httpClient.PostAsync(uri, sc).Result;
                    //if (httpResponseMessage.IsSuccessStatusCode)
                    //{
                    //    apiResult = httpResponseMessage.Content.ReadAsStringAsync().Result;
                    //}
                    apiResult = httpResponseMessage.Content.ReadAsStringAsync().Result;
                    LogEvent.WebAPILoger.Debug("SendControl apiResult = " + apiResult);
                }
            }

            var jsonResult = JObject.Parse(apiResult);
            if (jsonResult.GetValue("code").ToString() == ((int)EnumAPIResult.Succeed).ToString())
            {
                //LogEvent.WebAPILoger.Info("SendControl SUCCESS");
                result = true;
            }
            else
            {
                LogEvent.WebAPILoger.Error("SendControl ERROR, error message = " + jsonResult.GetValue("error").ToString(Newtonsoft.Json.Formatting.None));
            }

            return result;
        }

        private bool StartCharge(ChargeApplyInfo apply)
        {
            var uri = ControlAPI + "StartCharge";
            //LogEvent.WebAPILoger.Info("StartCharge 开始充电指令");
            var result = SendControl(apply, uri);

            return result;
        }

        internal bool PostStop(string userToken, string deviceCode)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            bool result = false;
            //反向对接判断状态是不是反向的设备
            //if (new PayLogic().GetEquipAuth(deviceCode).SuccStat == 0)
            RedisHelper reverse = new RedisHelper((int)EnumUserRedisNum.Reserve);
            string reverseDeviceCode = reverse.HashGetString(RedisKey.ReverseGunMap, deviceCode);
            if (!string.IsNullOrEmpty(reverseDeviceCode))
            {
                List<WXDeviceInfo> wxDeviceInfo = NewGetDeviceInfoByUserToken(userToken);
                var stopDevice = wxDeviceInfo.FirstOrDefault(x => x.DeviceCode == deviceCode);
                if (stopDevice != null)
                {
                    WXDeviceInfo device = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);
                    result = new PayLogic().ChargeStop(deviceCode, device.OrderID);
                }
            }
            else
            {
                //从userToken查询在哪一个设备上充电
                List<WXDeviceInfo> wxDeviceInfo = NewGetDeviceInfoByUserToken(userToken);
                var stopDevice = wxDeviceInfo.FirstOrDefault(x => x.DeviceCode == deviceCode);
                if (stopDevice != null)
                {
                    // 保存平台停止的原因 用户界面停止
                    redis.HashSet(RedisKey.EndResonDescription, stopDevice.OrderID, 992);

                    result = StopCharge(stopDevice);
                }
            }

            return result;
        }
        internal bool PostStop(string userToken)
        {
            LogEvent.WebAPILoger.Debug($"ChargeLogic-PostStop, userToken={userToken}");

            //从userToken查询在哪一个设备上充电
            WXDeviceInfo wxDeviceInfo = GetDeviceInfoByUserToken(userToken);

            // 保存平台停止的原因 用户界面停止
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            redis.HashSet(RedisKey.EndResonDescription, wxDeviceInfo.OrderID, 992);
            return StopCharge(wxDeviceInfo);
        }

        public bool ManualCloseOrder(HistoryOrder order)
        {
            bool result = false;
            order.ordertype = 3;
            order.endtime = DateTime.Now;
            DynamicParameters param = new DynamicParameters();
            param.Add("@devicecode", order.devicecode);
            param.Add("@ordertype", order.ordertype);
            param.Add("@orderid", order.orderid);
            param.Add("@userid", order.userid);
            param.Add("@connecttime", order.connecttime);
            param.Add("@commandtime", order.commandtime);
            param.Add("@starttime", order.starttime);
            param.Add("@endtime", order.endtime);
            param.Add("@startsoc", order.startsoc);
            param.Add("@endsoc", order.endsoc);
            param.Add("@forwardpower", order.forwardpower);
            param.Add("@reversepower", order.reversepower);
            param.Add("@stationcode", order.stationcode);
            param.Add("@canid", order.canid);
            param.Add("@cardid", order.cardid);
            param.Add("@vin", order.vin);
            param.Add("@chargetype", order.chargetype);
            param.Add("@limitpower", order.limitpower);
            param.Add("@chargeway", order.chargeway);
            param.Add("@endreason", order.endreason);
            param.Add("@power", order.power);
            param.Add("@chargestrategy", order.chargestrategy);
            param.Add("@chargepower", order.chargepower);
            param.Add("@dischargestrategy", order.dischargestrategy);
            param.Add("@dischargepower", order.dischargepower);
            param.Add("@dispower", order.dispower);
            param.Add("@chargecost", order.chargecost);
            param.Add("@dischargecost", order.dischargecost);
            param.Add("@servicecost", order.servicecost);

            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var spName = "ManualCloseOrder";
                int affectedRowNum = connection.Execute(spName, param, null, null, CommandType.StoredProcedure);
                if (affectedRowNum == 2)
                {
                    RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.RealData, RedisConnectionString);
                    redis.HashSet(RedisKey.ActiveOrder, order.devicecode, order);
                    result = true;
                }
            }

            return result;
        }

        //停止
        public Boolean StopCharge(WXDeviceInfo wxDeviceInfo)
        {

            var stopResult = false;
            if (wxDeviceInfo != null && wxDeviceInfo.GunState != EnumGunStatus.离线)
            {
                var apply = new ChargeApplyInfo();
                apply.Arrearage = 0;
                apply.ChargeCount = wxDeviceInfo.ChargeStrategy.Count;
                apply.ChargeList = wxDeviceInfo.ChargeStrategy;
                apply.ChargeType = (int)EnumChargeType.立即充电;
                apply.CommonBalance = 1;
                apply.ControlID = 5003;
                apply.DeviceCode = wxDeviceInfo.DeviceCode;
                apply.DischargeCount = 0;
                apply.DischargeList = new List<StrategyInfo>();
                apply.MarkID = wxDeviceInfo.OrderID;
                apply.OrderID = wxDeviceInfo.OrderID;
                apply.StationCode = wxDeviceInfo.StationCode;
                apply.UserID = wxDeviceInfo.Openid;
                apply.UserName = wxDeviceInfo.NickName;
                apply.Value = 0;
                apply.ValueDes = "";

                stopResult = StopCharge(apply);

            }

            return stopResult;
        }


        public List<ChargeState> NewGetDeviceStatusByUserToken(string userToken)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            List<ChargeState> chargeStateList = new List<ChargeState>();
            var openid = redis.HashGetString(RedisKey.Token, userToken);

            if (!string.IsNullOrEmpty(userToken))
            {
                try
                {

                    List<WXDeviceInfo> wxDeviceInfo = NewGetDeviceInfoByUserToken(userToken);
                    List<string> openidList = new List<string>();
                    List<string> arr = new List<string>();
                    string phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
                    if (openid.Length == 11)
                    {

                        phone = openid;
                    }

                    //为了兼容app通过微信支付领充openid,小马openid与微信openid兼容的问题 app openid为11位，在chargingdevice中找正在充电的设备 将两个opeid 关联
                    if (!string.IsNullOrEmpty(phone))
                    {
                        using (IDbConnection conn = WechatUtility.CreateDBConnection())
                        {
                            string sql = $"SELECT openid FROM lco_wxcustomer WHERE phone = '{phone}'";
                            openidList = conn.Query<string>(sql, conn).ToList();
                        }
                    }
                    openidList.Add(openid);
                    openidList.Distinct();
                    if (wxDeviceInfo != null)
                    {
                        foreach (var item in wxDeviceInfo)
                        {
                            if (((item.Openid.Length >= 28 && (openid == item.Openid.Substring(item.Openid.Length - 28, 28)) || openidList.Contains(openid)) || (openid.Length == 11 && (openid == item.Openid || openidList.Contains(item.Openid)))) && (item.GunState == EnumGunStatus.充电中 || item.GunState == EnumGunStatus.暂停 || item.GunState == EnumGunStatus.离线))
                                chargeStateList.Add(ConvertWXDeviceInfoToChargeState(item));
                        }
                    }
                }
                catch (Exception e)
                {
                    LogEvent.WebAPILoger.Error($"NewGetDeviceStatusByUserToken 无对应充电中设备！userToken={userToken},openid ={openid}", e);
                }
            }

            return chargeStateList;
        }

        internal int GetDeviceStatusByUserToken(string userToken)
        {
            int result = 0;
            if (!string.IsNullOrEmpty(userToken))
            {
                var wxDeviceInfo = GetDeviceInfoByUserToken(userToken);
                if (wxDeviceInfo == null)
                {
                    return -1;
                }
                if ((wxDeviceInfo.OrderState == EnumWXOrderState.充电中 && wxDeviceInfo.GunState == EnumGunStatus.充电中)
                    || (wxDeviceInfo.OrderState == EnumWXOrderState.充电中 && wxDeviceInfo.GunState == EnumGunStatus.启动中 && DateTime.Now.Subtract(wxDeviceInfo.LastUpdateTime).TotalMinutes <= 3))
                {
                    result = 99;
                }
                else
                {
                    result = (int)wxDeviceInfo.GunState;
                }
            }

            return result;
        }

        public ChargeState ConvertWXDeviceInfoToChargeState(WXDeviceInfo wxdeviceInfo)
        {

            ChargeState chargestate = new ChargeState();
            chargestate.GunState = (int)wxdeviceInfo.GunState;

            chargestate.DeviceCode = wxdeviceInfo.DeviceCode.ToString();
            chargestate.SOC = wxdeviceInfo.SOC;
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            //LCO_Station station = redis.HashGet<LCO_Station>(RedisKey.Station, new PayLogic().GetStationId(wxdeviceInfo.DeviceCode));
            LCO_Station station = redis.HashGet<LCO_Station>(RedisKey.Station, WechatUtility.GetStationId(wxdeviceInfo.DeviceCode));
            if (station != null)
            {
                chargestate.stationname = station.StationName;
            }


            chargestate.remainingTime = calculateRemainTime(wxdeviceInfo.DeviceCode);
            return chargestate;

        }

        public string calculateRemainTime(string deviceCode)
        {
            double soc = 0;
            double power = 0; //充电功率
            double voltage = 0; //电池额定电压
            double volume = 0;//电池容量

            TimeSpan retime = new TimeSpan();
            string remining = "";
            RedisHelper redis13 = new RedisHelper((int)EnumUserRedisNum.BMS, RedisConnectionString);
            try
            {
                Gunsignal gunsignal = redis13.HashGet<Gunsignal>(RedisKey.ChargeProcessData, deviceCode);
                if (gunsignal != null && gunsignal.signalList.Any())
                {
                    foreach (var item in gunsignal.signalList)
                    {
                        switch (item.signalid)
                        {
                            case 5702: soc = item.signalvalue.LastOrDefault(); break;
                            case 5710: volume = item.signalvalue.LastOrDefault(); break;
                            case 5306: power = item.signalvalue.LastOrDefault(); break;
                            case 5711: voltage = item.signalvalue.LastOrDefault(); break;
                            default:
                                break;
                        }

                    }
                }

                if (soc != 0 && power != 0 && volume != 0 && voltage != 0)
                {
                    double time = ((volume * volume / 1000) * (100 - soc) / 100) / power;
                    int totalseconds = (int)(time * 60 * 60);
                    retime = new TimeSpan(0, 0, totalseconds);
                    remining = retime.Hours + "时" + retime.Minutes + "分";
                }
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($@"获取bms出错　ｐａｒａｍ＝{deviceCode}，{e.ToString()}");
            }


            return remining;

        }

        internal int NewGetDeviceStatus(string userToken, string deviceCode)
        {
            #region 替换二维码字符串
            deviceCode = deviceCode.Replace("%2F", "/").Replace("%3A", ":").Replace("%7C", "|").Split('/').Last();
            #endregion
            #region 体验扫码启动Demo
            if (deviceCode.Length >= 15 && deviceCode.Substring(0, 15).Equals(DemoQRCode))
            {
                return 2;//连接状态
            }
            #endregion

            #region 测试二维码
            if (deviceCode.Length >= 15 && deviceCode.Substring(0, 15).Equals(TestQRCode))
            {
                return 2;//连接状态
            }
            #endregion
            //根据设备code确定此设备当前是否可用
            if (string.IsNullOrEmpty(deviceCode) || deviceCode == "undefined")
            {
                return (int)Common.EnumGunStatus.不存在;
            }
            //过滤随意扫的内容，减轻后端压力 正确的devicecode示例：6101120004:0001|181
            if (deviceCode.Length < 17 || deviceCode.Length > 19 || !deviceCode.Contains("|"))
            {
                LogEvent.WebAPILoger.Debug($"GetDeviceStatus Invalid deviceCode = {deviceCode}。");
                return (int)EnumGunStatus.不存在;
            }
            GunInfo gunInfo = GetRealDataByDeviceAndSignal(deviceCode, null);
            gunInfo = gunInfo == null ? new GunInfo() : gunInfo;

            return gunInfo.GunStatus;

        }

        public int GetGunStatus(string devicecode)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Struct, RedisConnectionString);

            var deviceInfo = redis.HashGet<cfg_device>(RedisKey.DeviceByDeviceCode, devicecode);

            GunStatusInfo result = new GunStatusInfo();
            if (deviceInfo != null)
            {
                double? connectStatus = GetRealDataValue(deviceInfo.deviceid, EnumSignalID.枪_连接确认开关状态);
                if (connectStatus == null)
                    return 99999;

                double? chargeStatus = GetRealDataValue(deviceInfo.deviceid, EnumSignalID.枪_充电机工作状态);
                if (chargeStatus == null)
                    return 99999;

                result.status_type = GetGunStatusValue((int)connectStatus.Value, (int)chargeStatus.Value);
            }
            else
            {
                return -99999;
            }
            return result.status_type;
        }
        static public int GetGunStatusValue(int connectStatus, int chargeStatus)
        {
            if (connectStatus == (int)EnumConnectStatus.断开)
            {
                if (chargeStatus == (int)EnumChargeStatus.离线)
                    return (int)EnumGunStatus.离线;
                else if (chargeStatus == (int)EnumChargeStatus.故障)
                    return (int)EnumGunStatus.故障;
                else
                    return (int)EnumGunStatus.未插枪;
            }

            switch ((EnumChargeStatus)chargeStatus)
            {
                case EnumChargeStatus.充电:
                    return (int)EnumGunStatus.充电中;
                case EnumChargeStatus.启动中:
                    return (int)EnumGunStatus.启动中;
                case EnumChargeStatus.放电:
                    return (int)EnumGunStatus.放电;
                case EnumChargeStatus.离线:
                    return (int)EnumGunStatus.离线;
                case EnumChargeStatus.故障:
                    return (int)EnumGunStatus.故障;
                case EnumChargeStatus.暂停:
                    return (int)EnumGunStatus.暂停;

                default:
                    return (int)EnumGunStatus.连接;
            }

        }

        static public double? GetRealDataValue(string deviceID, EnumSignalID signalID)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.RealData, RedisConnectionString);
            string index = String.Format("{0}|{1}", deviceID, (int)signalID);
            ServiceRealDataInfo realDataInfo = redis.HashGet<ServiceRealDataInfo>(RedisKey.RealDataList, index);

            if (realDataInfo == null)
                return null;
            else
                return realDataInfo.Value;
        }
        internal int GetDeviceStatus(string userToken, string deviceCode)
        {
            //根据设备code确定此设备当前是否可用
            if (string.IsNullOrEmpty(deviceCode) || deviceCode == "undefined")
            {
                return (int)Common.EnumGunStatus.不存在;
            }
            //过滤随意扫的内容，减轻后端压力 正确的devicecode示例：6101120004:0001|181
            if (deviceCode.Length < 17 || deviceCode.Length > 19 || !deviceCode.Contains("|"))
            {
                LogEvent.WebAPILoger.Debug($"GetDeviceStatus Invalid deviceCode = {deviceCode}。");
                return (int)EnumGunStatus.不存在;
            }

            var deviceStatus = GetDeviceStatusByUserToken(userToken);
            if (deviceStatus == 99)
            {
                return deviceStatus;
            }

            #region 体验扫码启动Demo
            if (deviceCode.Length >= 15 && deviceCode.Substring(0, 15).Equals(DemoQRCode))
            {
                return 2;//连接状态
            }
            #endregion

            #region 测试二维码
            if (deviceCode.Length >= 15 && deviceCode.Substring(0, 15).Equals(TestQRCode))
            {
                return 2;//连接状态
            }
            #endregion

            GunInfo gunInfo = GetRealDataByDeviceAndSignal(deviceCode, null);
            gunInfo = gunInfo == null ? new GunInfo() : gunInfo;

            return gunInfo.GunStatus;

        }
        //获取桩下对应的枪
        public List<string> GunOfDevice(string device)
        {
            List<string> GunName = new List<string>();
            if (device.StartsWith("WLwdss"))
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Protocol, RedisConnectionString);
                string GetDevice = redis.HashGetString(RedisKey.MapDeviceStation, device);
                if (GetDevice != null)
                {
                    device = GetDevice;
                }
                else
                {
                    GunName.Add(device);
                    return GunName;
                }
            }

            //小桔二维码的特征
            if (device.Length==14 && !device.Contains("|"))
            {
                var devicetemp = string.Format("{0}|{1}", device.Substring(0, 12), device.Substring(12, 2));
                RedisHelper reverse = new RedisHelper((int)EnumUserRedisNum.Reserve, RedisConnectionString);
                string reverseDeviceCode = reverse.HashGetString(RedisKey.ReverseGunMap, devicetemp);
                if (!string.IsNullOrEmpty(reverseDeviceCode))
                {
                    //如果存在映射关系，则证明是小桔的桩，用加了|的替换原来的。
                    device = devicetemp;
                }
            }

            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var sql = $@"SELECT DISTINCT keycode FROM cfg_devicegroup WHERE keycode LIKE '{device}%' AND layer='5'  ";
                GunName = SqlMapper.Query<string>(connection, sql).ToList();
            }
            return GunName;

        }

        /// <summary>
        /// 判定是不是vin设备
        /// </summary>
        /// <param name="device"></param>
        /// <returns></returns>
        public bool ISVIN(string device)
        {
            bool ISVIN = false;
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            List<string> VinList = redis.HashGet<List<string>>(RedisKey.VinDeviceList, "1");
            if (VinList.Contains(device))
            {
                ISVIN = true;
            }
            return ISVIN;
        }
        /// <summary>
        /// 保存vin启动相关信息
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <param name="startTime"></param>
        /// <param name="usertoken"></param>
        /// <returns></returns>
        public bool PostVinTimeInfo(string deviceCode, string startTime, string usertoken)
        {
            bool insertSuccess = false;
            try
            {
                VinChargeTime info = new VinChargeTime();
                RedisHelper redis = new RedisHelper((int)(EnumUserRedisNum.WeiXin));

                string openid = GetOpenIDByUserToken(usertoken);
                string phone = redis.HashGet<string>(RedisKey.UserAndPhone, openid);
                ServiceRealDataInfo gunRealDataVin = GetGunRealState(deviceCode, EnumSignalID.枪_汽车VIN码);
                if (gunRealDataVin == null || string.IsNullOrEmpty(gunRealDataVin.Meanings))
                {
                    LogEvent.WebAPILoger.Warn($"未获取到Vin号，devocecode={deviceCode}，openid={openid},phone = {phone}");
                    return false;
                }



                if (!string.IsNullOrEmpty(openid) && !string.IsNullOrEmpty(phone))
                {
                    info.openid = openid;
                    info.phone = phone;
                }
                info.StartTime = DateTime.Parse(startTime);
                info.vinID = gunRealDataVin.Meanings;
                string stationid = WechatUtility.GetStationId(deviceCode);
                info.companyID = redis.HashGet<LCO_Station>(RedisKey.Station, stationid).CompanyID;
                info.deviceCode = deviceCode;
                info.chargeCommand = false;



                List<VinChargeTime> vinchargeInfo = redis.HashGet<List<VinChargeTime>>(RedisKey.VinChargeTime, "1");

                if (vinchargeInfo == null)
                {
                    vinchargeInfo = new List<VinChargeTime>();
                }

                vinchargeInfo.RemoveAll(p => p.deviceCode == deviceCode);
                vinchargeInfo.Add(info);

                redis.HashSet<List<VinChargeTime>>(RedisKey.VinChargeTime, "1", vinchargeInfo);
                info.SendedTime = DateTime.Now.AddDays(-1).ToShortDateString();
                info.VinTermlyStart = Convert.ToDateTime(startTime).TimeOfDay;
                redis.HashSet<VinChargeTime>(RedisKey.VinChargeTime, info.vinID, info);
                redis.HashSet<string>(RedisKey.VinAndOpeid, openid, info.vinID);
                insertSuccess = true;
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"保存Vin启动信息失败！{e}");
            }
            return insertSuccess;

        }
        /// <summary>
        /// 获取用户当前设置的vin时间
        /// </summary>
        /// <param name="usertoken"></param>
        /// <returns></returns>
        public dynamic GetVinTimeInfo(string usertoken)
        {
            VinChargeTime info = new VinChargeTime();

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            string openid = WechatUtility.GetOpenIDByUserToken(usertoken);
            if (string.IsNullOrEmpty(openid))
            {
                return null;
            }

            string vinId = redis.HashGetString(RedisKey.VinAndOpeid, openid);
            if (string.IsNullOrEmpty(vinId))
            {
                return null;
            }

            info = redis.HashGet<VinChargeTime>(RedisKey.VinChargeTime, vinId);
            if (info == null)
            {
                return null;
            }
            else
            {
                return info.VinTermlyStart;
            }
        }

        public bool DeleteVinTimeInfo(string userToken)
        {
            bool result = false;
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);

                string openid = WechatUtility.GetOpenIDByUserToken(userToken);

                List<VinChargeTime> data = redis.HashGet<List<VinChargeTime>>(RedisKey.VinChargeTime, "1");
                data.RemoveAll(p => p.openid == openid);
                redis.HashSet<List<VinChargeTime>>(RedisKey.VinChargeTime, "1", data);
                string Vinid = redis.HashGetString(RedisKey.VinAndOpeid, openid);
                redis.HashDelete(RedisKey.VinChargeTime, Vinid);
                result = true;
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error(e);
            }
            return result;
        }

        private ServiceRealDataInfo GetGunRealState(string devicecode, EnumSignalID signalID)
        {
            RedisHelper redisstuct = new RedisHelper((int)EnumUserRedisNum.Struct, RedisConnectionString);
            cfg_device deviceInfo = redisstuct.HashGet<cfg_device>(RedisKey.DeviceByDeviceCode, devicecode);
            ServiceRealDataInfo realdata = new ServiceRealDataInfo();
            if (deviceInfo != null)
            {
                string index = String.Format("{0}|{1}", deviceInfo.deviceid, (int)signalID);
                RedisHelper redisreal = new RedisHelper((int)EnumUserRedisNum.RealData, RedisConnectionString);
                realdata = redisreal.HashGet<ServiceRealDataInfo>(RedisKey.RealDataList, index);
            }
            return realdata;
        }
        #endregion

        #region 微信相关
        public string Login(string code)
        {
            string userToken = string.Empty;
            try
            {
                //构造获取openid的url
                WxPayData data = new WxPayData();
                data.SetValue("appid", WxPayConfig.MINA_APPID);
                data.SetValue("secret", WxPayConfig.MINA_APPSECRET);
                data.SetValue("js_code", code);
                data.SetValue("grant_type", "authorization_code");
                string url = "https://api.weixin.qq.com/sns/jscode2session?" + data.ToUrl();

                //请求url以获取数据
                string apiResult = HttpService.Get(url);
                //LogEvent.WebAPILoger.Debug(this.GetType().ToString(), "GetOpenidAndAccessTokenFromCode response : " + apiResult);

                var jsonData = JObject.Parse(apiResult);
                if (jsonData.GetValue("openid") != null)
                {
                    //以openid为Key，将openid、assess_key信息存入redis
                    userToken = Guid.NewGuid().ToString("N");
                    var openid = jsonData.GetValue("openid").ToString();
                    SaveTokenToRedis(userToken, openid);
                }
                else
                {
                    LogEvent.WebAPILoger.Error("ChargeLogic - Login - apiResult:" + apiResult);
                }

            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("ChargeLogic - Login", ex);
            }

            return userToken;
        }

        private void SaveTokenToRedis(string userToken, string openid)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);

            var oldToken = redis.HashGetString(RedisKey.TokenReverse, openid);
            if (!string.IsNullOrEmpty(oldToken))
            {
                redis.HashDelete(RedisKey.Token, oldToken);
            }
            redis.HashSet(RedisKey.TokenReverse, openid, userToken);
            redis.HashSet(RedisKey.Token, userToken, openid);
            redis.HashSet(RedisKey.UserAndPhone, openid, openid);
        }

        public string WXMINAPay(WXMINAPayParameter param)
        {
            var openid = GetOpenIDByUserToken(param.userToken);

            var payData = GetUnifiedOrderResult(openid, param);

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            redis.HashSet(RedisKey.WXPrepayID, openid, payData.GetValue("prepay_id"));

            var parameters = GetJsApiParameters(payData);

            return parameters;
        }

        public string WXMINAPayNotify()
        {
            ResultNotify resultNotify = new ResultNotify();
            //获取微信通知数据
            WxPayData notifyData = resultNotify.GetNotifyData();
            //检查订单合法性
            var orderCheckResult = resultNotify.ProcessNotify2(notifyData);

            if (orderCheckResult.GetValue("return_code").ToString() == "SUCCESS")
            {
                var openid = notifyData.GetValue("openid").ToString();
                var total_fee = int.Parse(notifyData.GetValue("total_fee").ToString());
                var nickName = notifyData.GetValue("attach").ToString();
                var deviceCode = notifyData.GetValue("device_info").ToString();
                var stationCode = deviceCode.Split('|')[0];
                //var jkCode = deviceCode.Split('|')[0];
                //var stationCode = jkCode.Length >= 10 ? jkCode.Substring(0, 10) : jkCode;
                var transaction_id = notifyData.GetValue("transaction_id").ToString();
                var out_trade_no = notifyData.GetValue("out_trade_no").ToString();
                var time_end = long.Parse(notifyData.GetValue("time_end").ToString());

                if (CreateRedisOrder(openid, total_fee, nickName, deviceCode, stationCode, transaction_id, out_trade_no, time_end, EnumPaymentType.小程序微信))
                {
                    return orderCheckResult.ToXml();
                }
            }

            return "";
        }

        public string Refund(WXRefund refundParameter)
        {
            string resultString = "";

            switch (refundParameter.PaymentType)
            {
                case EnumPaymentType.小程序微信:
                    WxPayData refundResult = WxPayAPI.Refund.Run(refundParameter.Transaction_id, refundParameter.Out_trade_no, refundParameter.Total_fee, refundParameter.Refund_fee);
                    resultString = refundResult.ToJson();

                    break;

                case EnumPaymentType.银商支付宝:
                    resultString = ALIRefund(refundParameter);

                    break;
                case EnumPaymentType.威富通微信:
                    //resultString = WFTRefund(refundParameter);


                    break;

                default:
                    break;
            }

            return resultString;
        }

        private bool CreateRedisOrder(string openid, int total_fee, string nickName, string deviceCode,
            string stationCode, string transaction_id, string out_trade_no, long time_end, EnumPaymentType paymentType)
        {
            bool result = false;

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var oldDeviceInfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);
            string prepayID = GetTemplateMessageUserID(openid, transaction_id, paymentType, redis);

            var newDeviceInfo = CreateCurrentDeviceInfo(openid, total_fee, nickName, deviceCode, stationCode, transaction_id, out_trade_no, time_end, paymentType, prepayID);

            //根据回调的数据发起充电请求
            //微信和支付宝有可能多次通知
            if (oldDeviceInfo == null)
            {
                result = ChargeAndSaveOrderInfo(newDeviceInfo, redis);
            }
            else
            {
                if (oldDeviceInfo.Time_end >= newDeviceInfo.Time_end)//同一个设备支付时间一样或比当前还早，可以认为是重复通知
                {
                    LogEvent.WebAPILoger.Debug($"WXMINAPayNotify 再次通知 if(wxDeviceInfo.Time_end >= newDeviceInfo.Time_end), wxDeviceInfo.Time_end={oldDeviceInfo.Time_end}, newDeviceInfo.Time_end={newDeviceInfo.Time_end}, nickName={newDeviceInfo.NickName}，total_fee={newDeviceInfo.Total_fee}，transaction_id={newDeviceInfo.Transaction_id}");
                    result = true;
                }
                else if (oldDeviceInfo.Time_end < time_end)
                {
                    if (oldDeviceInfo.OrderState != EnumWXOrderState.充电中)
                    {
                        //第一次通知，枪在redis中有缓存，则覆盖上一次充电记录
                        result = ChargeAndSaveOrderInfo(newDeviceInfo, redis);
                    }
                    else
                    {
                        //已经充电中的桩，再次被付款，则退款
                        LogEvent.WebAPILoger.Info("MINAPayNotify:, 已经充电中的桩，再次被付款，退款");
                        WXRefund refundParam = new WXRefund();
                        refundParam.Transaction_id = transaction_id;
                        refundParam.Out_trade_no = out_trade_no;
                        refundParam.Total_fee = total_fee;
                        refundParam.Refund_fee = total_fee;
                        refundParam.PaymentType = paymentType;
                        var refundResult = Refund(refundParam);
                        LogEvent.WebAPILoger.Info($"MINAPayNotify:Refund Result = {refundResult}");
                        result = true;
                    }
                }
            }

            return result;
        }

        private static string GetTemplateMessageUserID(string openid, string transaction_id, EnumPaymentType paymentType, RedisHelper redis)
        {
            string prepayID = string.Empty;
            switch (paymentType)
            {
                case EnumPaymentType.小程序微信:
                    prepayID = redis.HashGetString(RedisKey.WXPrepayID, openid);
                    break;
                case EnumPaymentType.银商支付宝:
                    prepayID = transaction_id;
                    break;
                case EnumPaymentType.威富通微信:
                    break;
                default:
                    break;
            }

            return prepayID;
        }

        private WXDeviceInfo CreateCurrentDeviceInfo(string openid, int total_fee, string nickName, string deviceCode, string stationCode,
            string transaction_id, string out_trade_no, long time_end, EnumPaymentType paymentType, string prepayID)
        {
            var deviceInfo = new WXDeviceInfo();
            RedisHelper redis5 = new RedisHelper((int)EnumUserRedisNum.Struct);
            var orderID = string.Format("{0}{1}", DateTime.Now.ToString("yyMMddHHmmssfff"), new Random().Next(100, 999));

            string stationid = WechatUtility.GetStationId(deviceCode);
            List<StrategyInfo> chargeStrategy = GetChargeStrategy(stationid);
            double servicePrice = GetServicePrice(stationid);

            deviceInfo.VIP = 0;
            deviceInfo.Openid = openid;
            deviceInfo.DeviceType = redis5.HashGet<cfg_device>(RedisKey.DeviceByDeviceCode, deviceCode).devicetype;
            //deviceInfo.DeviceType = int.Parse(deviceCode.Split('|')[1]) > 180 ? EnumSubDeviceType.直流充电枪 : EnumSubDeviceType.交流充电枪;//直流canID从181到230
            deviceInfo.StationCode = stationCode;
            deviceInfo.DeviceCode = deviceCode;
            deviceInfo.NickName = nickName;
            deviceInfo.OrderState = EnumWXOrderState.未开始;
            deviceInfo.GunState = EnumGunStatus.启动中;
            deviceInfo.Total_fee = total_fee;
            deviceInfo.RemainingFee = total_fee * 10;
            deviceInfo.Transaction_id = transaction_id;
            deviceInfo.Out_trade_no = out_trade_no;
            deviceInfo.OrderID = orderID;
            deviceInfo.ChargeStrategy = chargeStrategy;
            deviceInfo.StartTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            deviceInfo.ServiceFee = servicePrice;
            deviceInfo.Time_end = time_end;
            deviceInfo.PaymentType = paymentType;
            deviceInfo.Voltage = "";
            deviceInfo.Current = "";
            deviceInfo.Power = "";
            deviceInfo.Duration = "";
            deviceInfo.PrepayID = prepayID;
            deviceInfo.ForwardPower = -0.001; // 初始电表读数为负数，方便区分是电表初始读数真是0，还是没有采集上来呢

            return deviceInfo;
        }

        private bool ChargeAndSaveOrderInfo(WXDeviceInfo newDeviceInfo, RedisHelper redis)
        {
            #region 体验扫码启动Demo
            if (newDeviceInfo != null && newDeviceInfo.DeviceCode.Substring(0, 15).Equals(DemoQRCode))
            {
                List<StrategyInfo> chargeStrategyTest = new List<StrategyInfo>();
                chargeStrategyTest.Add(new StrategyInfo() { StartTime = new STimeInfo(0, 0, 0, true), EndTime = new STimeInfo(23, 59, 59, true), Price = 2000, Valid = true });
                newDeviceInfo.ChargeStrategy = chargeStrategyTest;
                newDeviceInfo.ServiceFee = 1;

                var chargeResultTest = true;
                if (chargeResultTest == true)
                {
                    redis.HashSet<string>("OpenID", newDeviceInfo.Openid, newDeviceInfo.DeviceCode);


                    newDeviceInfo.DeviceType = (int)EnumSubDeviceType.直流充电枪;
                    newDeviceInfo.OrderState = EnumWXOrderState.充电中;
                    newDeviceInfo.GunState = EnumGunStatus.充电中;
                    newDeviceInfo.Voltage = "888";
                    newDeviceInfo.Current = "88";
                    newDeviceInfo.Power = "8888";
                    newDeviceInfo.Duration = "0小时8分钟";
                    newDeviceInfo.SOC = 88;
                    newDeviceInfo.PowerConsumption = 8.0;
                    SaveOrderInfo(newDeviceInfo, redis);

                    return true;
                }
            }
            #endregion

            var chargeResult = Charge(newDeviceInfo);

            if (chargeResult == true)
            {
                LogEvent.WebAPILoger.Debug($"PayNotify startChargeResult=真,deviceCode={newDeviceInfo.DeviceCode},nickName={newDeviceInfo.NickName},total_fee={newDeviceInfo.Total_fee},transaction_id={newDeviceInfo.Transaction_id},out_trade_no={newDeviceInfo.Out_trade_no},OrderID={newDeviceInfo.OrderID}");
                SaveOrderInfo(newDeviceInfo, redis);
                SendGunStatusMessage(newDeviceInfo.DeviceCode, EnumGunStatus.启动中);
                return true;
            }

            return false;
        }

        private void SendGunStatusMessage(string deviceCode, EnumGunStatus gunStatus)
        {
            EnumSubDeviceType deviceType = int.Parse(deviceCode.Split('|')[1]) > 180 ? EnumSubDeviceType.直流充电枪 : EnumSubDeviceType.交流充电枪;
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.RealData, RedisConnectionString);

            string message = $"{deviceCode}|{(int)gunStatus}|{(int)deviceType}";
            redis.ListRightPush(RedisKey.GunStatus, message);
        }

        private bool SaveOrderInfo(WXDeviceInfo newDeviceInfo, RedisHelper redis)
        {
            //
            redis.HashSet(RedisKey.OpenID, newDeviceInfo.Openid, newDeviceInfo.DeviceCode);

            //
            List<string> chargingDeviceCode = redis.HashGet<List<string>>(RedisKey.ChargingDevice, newDeviceInfo.Openid);
            if (chargingDeviceCode == null)
            {
                chargingDeviceCode = new List<string>();
            }

            chargingDeviceCode.Add(newDeviceInfo.DeviceCode);
            redis.HashSet(RedisKey.DeviceInfo, newDeviceInfo.DeviceCode, newDeviceInfo);
            redis.HashSet(RedisKey.ChargingDevice, newDeviceInfo.Openid, chargingDeviceCode);

            return true;
        }

        private bool Charge(WXDeviceInfo newDeviceInfo)
        {
            //启动充电
            ChargeApplyInfo apply = GetStartChargeApply(newDeviceInfo);
            var startChargeResult = StartCharge(apply);

            return startChargeResult;
        }

        private ChargeApplyInfo GetStartChargeApply(WXDeviceInfo newDeviceInfo)
        {
            var apply = new ChargeApplyInfo();
            apply.MarkID = newDeviceInfo.OrderID;
            apply.StationCode = newDeviceInfo.StationCode;
            apply.DeviceCode = newDeviceInfo.DeviceCode;
            apply.ControlID = 5001;
            apply.UserID = newDeviceInfo.Openid;
            apply.UserName = newDeviceInfo.NickName;
            apply.OrderID = newDeviceInfo.OrderID;
            apply.ChargeType = (int)EnumChargeType.立即充电;
            apply.Value = 0;
            apply.ValueDes = "";
            apply.Arrearage = 0;
            //设置金额 微信中的金额单位是分，协议中是厘
            apply.CommonBalance = newDeviceInfo.Total_fee * 10;
            //获取充电策略
            apply.ChargeList = newDeviceInfo.ChargeStrategy;
            apply.ChargeCount = newDeviceInfo.ChargeStrategy.Count;
            apply.DischargeList = new List<StrategyInfo>();
            apply.DischargeCount = apply.DischargeList.Count;

            return apply;
        }

        public string GetOpenIDByUserToken(string userToken)
        {
            if (string.IsNullOrEmpty(userToken))
            {
                return "";
            }

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var openid = redis.HashGetString(RedisKey.Token, userToken);

            return openid;
        }

        private List<WXDeviceInfo> NewGetDeviceInfoByUserToken(string userToken)
        {
            if (string.IsNullOrEmpty(userToken))
            {
                return new List<WXDeviceInfo>();
            }

            string openid = WechatUtility.GetOpenIDByUserToken(userToken);
            if (string.IsNullOrEmpty(openid))
            {
                return new List<WXDeviceInfo>();
            }

            List<WXDeviceInfo> wxDeviceInfo = new List<WXDeviceInfo>();
            List<string> openidList = new List<string>();
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            List<string> arr = new List<string>();
            string phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
            if (openid.Length == 11)
            {
                phone = openid;
            }

            //为了兼容app通过微信支付领充openid,小马openid与微信openid兼容的问题 app openid为11位，在chargingdevice中找正在充电的设备 将两个opeid 关联
            if (!string.IsNullOrEmpty(phone))
            {
                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {
                    string sql = $"SELECT openid FROM lco_wxcustomer WHERE phone = '{phone}'";
                    openidList = conn.Query<string>(sql, conn).ToList();
                }
            }
            openidList.Add(openid);
            openidList = openidList.Distinct().ToList();
            foreach (var item in openidList)
            {
                List<string> chargingdevice = redis.HashGet<List<string>>(RedisKey.ChargingDevice, item);
                if (chargingdevice != null)
                {
                    arr.AddRange(chargingdevice);
                }
            }

            foreach (var item in arr.Distinct())
            {
                WXDeviceInfo deviceinfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, item);
                if (deviceinfo==null)
                {
                    continue;
                }
                if (!openidList.Contains(deviceinfo.Openid))
                {
                    if (!string.IsNullOrEmpty(deviceinfo.Openid) && deviceinfo.Openid.Length >= 28 && deviceinfo.Openid.Split('|').Length > 1 && openidList.Contains(deviceinfo.Openid.Split('|')[1]))
                    {
                    }
                    else
                        continue;
                }

                var duration = DateTime.Now - DateTime.Parse(deviceinfo.StartTime.Replace('-', '/'));
                int gunstatus = GetGunStatus(deviceinfo.DeviceCode);
                if (gunstatus != 99999)
                {
                    deviceinfo.GunState = (EnumGunStatus)gunstatus;
                }
                
                string convert1 = string.Format("{0}时{1}分", (duration.Days * 24 + duration.Hours).ToString(), duration.Minutes.ToString());
                deviceinfo.Duration = convert1;
                wxDeviceInfo.Add(deviceinfo);
            }

            return wxDeviceInfo;
        }
        private WXDeviceInfo GetDeviceInfoByUserToken(string userToken)
        {
            if (string.IsNullOrEmpty(userToken))
            {
                return null;
            }

            WXDeviceInfo wxDeviceInfo = null;
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);

            var openid = redis.HashGetString(RedisKey.Token, userToken);
            if (string.IsNullOrEmpty(openid))
            {
                return null;
            }

            var deviceCode = redis.HashGetString(RedisKey.OpenID, openid);

            if (string.IsNullOrEmpty(deviceCode))
            {
                return null;
            }

            wxDeviceInfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);
            if (wxDeviceInfo == null || wxDeviceInfo.Openid != openid)
            {
                return null;
            }

            return wxDeviceInfo;
        }

        private WXDeviceInfo GetDeviceInfo(string deviceCode)
        {
            WXDeviceInfo wxDeviceInfo = null;
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            if (string.IsNullOrEmpty(deviceCode))
            {
                return null;
            }

            wxDeviceInfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);
            if (wxDeviceInfo == null)
            {
                return null;
            }

            return wxDeviceInfo;
        }

        /// <summary>
        /// 调用统一下单，获得下单结果
        /// </summary>
        /// <param name="openID"></param>
        /// <param name="stationCode"></param>
        /// <param name="deviceCode"></param>
        /// <param name="fee">下单费用，单位：分</param>
        /// <returns>统一下单结果</returns>
        /// <exception cref="">失败时抛异常WxPayException</exception>
        private WxPayData GetUnifiedOrderResult(string openID, WXMINAPayParameter parameter)
        {
            //统一下单
            WxPayData data = new WxPayData();
            //自定义参数，可以为终端设备号(门店号或收银设备ID)，PC网页或公众号内支付可以传"WEB"
            data.SetValue("device_info", parameter.deviceCode);
            data.SetValue("body", string.IsNullOrEmpty(ChargeLogic.WXBody) ? "西安特锐德-充电费用" : ChargeLogic.WXBody);
            data.SetValue("attach", parameter.nickName);
            data.SetValue("out_trade_no", WxPayApi.GenerateOutTradeNo());
            data.SetValue("total_fee", parameter.fee);
            data.SetValue("time_start", DateTime.Now.ToString("yyyyMMddHHmmss"));
            data.SetValue("time_expire", DateTime.Now.AddMinutes(10).ToString("yyyyMMddHHmmss"));
            data.SetValue("goods_tag", "test");
            data.SetValue("trade_type", "JSAPI");
            data.SetValue("openid", openID);

            LogEvent.WebAPILoger.Debug($"GetUnifiedOrderResult 统一下单.openid={openID}");

            WxPayData result = WxPayApi.UnifiedOrder(data);
            if (!result.IsSet("appid") || !result.IsSet("prepay_id") || result.GetValue("prepay_id").ToString() == "")
            {
                LogEvent.WebAPILoger.Error($"GetUnifiedOrderResult - UnifiedOrder response error! result = {result.ToPrintStr()}");
                throw new WxPayException("UnifiedOrder response error!");
            }

            return result;
        }

        /// <summary>
        /// 从统一下单成功返回的数据中获取微信浏览器调起jsapi支付所需的参数
        /// </summary>
        /// <param name="unifiedOrderResult"></param>
        /// <returns>微信浏览器调起JSAPI时的输入参数，json格式可以直接做参数用</returns>
        /// <remarks>
        /// </remarks>
        /// <![CDATA[        
        /// * 微信浏览器调起JSAPI时的输入参数格式如下：
        /// * {
        /// *   "appId" : "wx2421b1c4370ec43b",     //公众号名称，由商户传入     
        /// *   "timeStamp":" 1395712654",         //时间戳，自1970年以来的秒数     
        /// *   "nonceStr" : "e61463f8efa94090b1f366cccfbbb444", //随机串     
        /// *   "package" : "prepay_id=u802345jgfjsdfgsdg888",     
        /// *   "signType" : "MD5",         //微信签名方式:    
        /// *   "paySign" : "70EA570631E4BB79628FBCA90534C63FF7FADD89" //微信签名 
        /// * }
        /// ]]>
        /// <see cref="http://pay.weixin.qq.com/wiki/doc/api/jsapi.php?chapter=7_7"/>
        private string GetJsApiParameters(WxPayData unifiedOrderResult)
        {
            //LogEvent.WebAPILoger.Debug(this.GetType().ToString(), "JsApiPay::GetJsApiParam is processing...");

            WxPayData jsApiParam = new WxPayData();
            jsApiParam.SetValue("appId", unifiedOrderResult.GetValue("appid"));
            jsApiParam.SetValue("timeStamp", WxPayApi.GenerateTimeStamp());
            jsApiParam.SetValue("nonceStr", WxPayApi.GenerateNonceStr());
            jsApiParam.SetValue("package", "prepay_id=" + unifiedOrderResult.GetValue("prepay_id"));
            jsApiParam.SetValue("signType", "MD5");
            jsApiParam.SetValue("paySign", jsApiParam.MakeSign());

            string parameters = jsApiParam.ToJson();

            //LogEvent.WebAPILoger.Debug(this.GetType().ToString(), "Get jsApiParam : " + parameters);
            return parameters;
        }
        #endregion

        #region APP微信相关

        public WXAppPayReq WXAPPPay(WXMINAPayParameter param)
        {
            APP_User user = GetAPPUser(param.userToken);
            if (user == null)
            {
                return null;
            }
            var openid = user.loginid + "|" + user.nickname;

            var payData = GetAPPUnifiedOrderResult(openid, param);

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            redis.HashSet(RedisKey.WXPrepayID, openid, payData.GetValue("prepay_id"));

            return GetAPPPayReq(payData);
        }

        /// <summary>
        /// 调用统一下单，获得下单结果
        /// </summary>
        /// <param name="openID"></param>
        /// <param name="stationCode"></param>
        /// <param name="deviceCode"></param>
        /// <param name="fee">下单费用，单位：分</param>
        /// <returns>统一下单结果</returns>
        /// <exception cref="">失败时抛异常WxPayException</exception>
        private WxPayData GetAPPUnifiedOrderResult(string openID, WXMINAPayParameter parameter)
        {
            //统一下单
            WxPayData data = new WxPayData();
            //自定义参数，可以为终端设备号(门店号或收银设备ID)，PC网页或公众号内支付可以传"WEB"
            data.SetValue("appid", "wxf7ba7843323af584");
            data.SetValue("mch_id", "1513664691");
            data.SetValue("device_info", parameter.code);
            data.SetValue("body", string.IsNullOrEmpty(ChargeLogic.WXBody) ? "充电费用" : ChargeLogic.WXBody);
            data.SetValue("attach", openID);
            data.SetValue("out_trade_no", WxPayApi.GenerateOutTradeNo());
            data.SetValue("total_fee", parameter.fee);
            data.SetValue("spbill_create_ip", parameter.spbillCreateIp);
            data.SetValue("time_start", DateTime.Now.ToString("yyyyMMddHHmmss"));
            data.SetValue("time_expire", DateTime.Now.AddMinutes(10).ToString("yyyyMMddHHmmss"));
            data.SetValue("goods_tag", "test");
            data.SetValue("trade_type", "APP");
            //data.SetValue("openid", openID);

            //LogEvent.WebAPILoger.Debug($"GetAPPUnifiedOrderResult 统一下单.openid={openID}");

            WxPayData result = WxPayApi.UnifiedOrder(data, 1, 6);
            if (!result.IsSet("appid") || !result.IsSet("prepay_id") || result.GetValue("prepay_id").ToString() == "")
            {
                LogEvent.WebAPILoger.Error($"GetAPPUnifiedOrderResult - UnifiedOrder response error! result = {result.ToPrintStr()}");
                throw new WxPayException("UnifiedOrder response error!");
            }

            return result;
        }

        /// <summary>
        /// 从统一下单成功返回的数据中获取微信浏览器调起jsapi支付所需的参数
        /// </summary>
        /// <param name="unifiedOrderResult"></param>
        /// <returns>微信浏览器调起JSAPI时的输入参数，json格式可以直接做参数用</returns>
        /// <remarks>
        /// </remarks>
        /// <![CDATA[        
        /// * 微信浏览器调起JSAPI时的输入参数格式如下：
        /// * {
        /// *   "appId" : "wx2421b1c4370ec43b",     //公众号名称，由商户传入     
        /// *   "timeStamp":" 1395712654",         //时间戳，自1970年以来的秒数     
        /// *   "nonceStr" : "e61463f8efa94090b1f366cccfbbb444", //随机串     
        /// *   "package" : "prepay_id=u802345jgfjsdfgsdg888",     
        /// *   "signType" : "MD5",         //微信签名方式:    
        /// *   "paySign" : "70EA570631E4BB79628FBCA90534C63FF7FADD89" //微信签名 
        /// * }
        /// ]]>
        /// <see cref="http://pay.weixin.qq.com/wiki/doc/api/jsapi.php?chapter=7_7"/>
        private WXAppPayReq GetAPPPayReq(WxPayData unifiedOrderResult)
        {
            //LogEvent.WebAPILoger.Debug(this.GetType().ToString(), "JsApiPay::GetJsApiParam is processing...");

            string noncestr = WxPayApi.GenerateNonceStr();
            string timestamp = WxPayApi.GenerateTimeStamp();

            WxPayData jsApiParam = new WxPayData();
            jsApiParam.SetValue("appid", unifiedOrderResult.GetValue("appid"));
            jsApiParam.SetValue("partnerid", WxPayConfig.ANDROID_MCHID);
            jsApiParam.SetValue("prepayid", unifiedOrderResult.GetValue("prepay_id"));
            jsApiParam.SetValue("package", "Sign=WXPay");
            jsApiParam.SetValue("noncestr", noncestr);
            jsApiParam.SetValue("timestamp", timestamp);
            //jsApiParam.SetValue("sign", jsApiParam.MakeSign_APP(WxPayConfig.ANDROID_KEY));

            WXAppPayReq appPayReq = new WXAppPayReq();
            appPayReq.appId = unifiedOrderResult.GetValue("appid").ToString();
            appPayReq.partnerId = WxPayConfig.ANDROID_MCHID;
            appPayReq.prepayId = unifiedOrderResult.GetValue("prepay_id").ToString();
            appPayReq.packageValue = "Sign=WXPay";
            appPayReq.nonceStr = noncestr;
            appPayReq.timeStamp = timestamp;
            appPayReq.sign = jsApiParam.MakeSign_APP(WxPayConfig.ANDROID_KEY);

            //string parameters = jsApiParam.ToJson();

            Log.Debug("PayReq", appPayReq.toString());
            return appPayReq;
        }

        public string WXAPPPayNotify()
        {
            ResultNotify resultNotify = new ResultNotify();
            //获取微信通知数据
            WxPayData notifyData = resultNotify.GetNotifyData(WxPayConfig.ANDROID_KEY);
            //检查订单合法性
            var orderCheckResult = resultNotify.ProcessNotify(notifyData, WxPayConfig.ANDROID_KEY);

            Log.Debug("orderCheckResult", orderCheckResult.ToXml());
            if (orderCheckResult.GetValue("return_code").ToString() == "SUCCESS")
            {
                string phoneNumnAndNickName = notifyData.GetValue("attach").ToString();
                var phoneNum = phoneNumnAndNickName.Split('|')[0];
                var nickName = phoneNumnAndNickName.Split('|')[1];
                var openid = notifyData.GetValue("openid").ToString();
                var total_fee = int.Parse(notifyData.GetValue("total_fee").ToString());
                var transaction_id = notifyData.GetValue("transaction_id").ToString();
                var out_trade_no = notifyData.GetValue("out_trade_no").ToString();
                var paymentTime = DateTime.ParseExact(notifyData.GetValue("time_end").ToString(), "yyyyMMddHHmmss", null).ToString();

                if (SaveRechargeRecord(phoneNum, nickName, openid, openid, total_fee, transaction_id, out_trade_no, paymentTime, EnumPaymentType.小程序微信))
                {
                    return orderCheckResult.ToXml();
                }
            }
            return "";
        }

        #endregion

        #region 局站
        internal IEnumerable<LCO_Station> GetStation(string userToken, double longitude, double latitude)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var allStations = redis.HashValues<LCO_Station>(RedisKey.Station);

            return allStations;

            ////var openid = GetOpenIDByUserToken(userToken);

            //IEnumerable<dynamic> stations = new List<dynamic>();
            //using (IDbConnection connection = CreateDBConnection())
            //{
            //    //TODO:经纬度没有 考虑增加权限分组等
            //    var sql = $@"SELECT keycode,stationname,longitude AS E,latitude AS N FROM cfg_station";
            //    stations = SqlMapper.Query<dynamic>(connection, sql);
            //}

            //return stations;
        }

        internal IEnumerable<LCO_Station> GetStationDetail(string stationCode, string usertoken = null)
        {
            if (string.IsNullOrEmpty(stationCode))
            {
                return null;
            }

            var stations = new List<LCO_Station>();
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);

            var station = redis.HashGet<LCO_Station>(RedisKey.Station, stationCode);
            if (station==null)
            {
                return null;
            }

            //增加场站图片信息
            if (Directory.Exists(System.Web.HttpContext.Current.Server.MapPath($"~/stationImages/{station.StationId}/")) == true)
            {
                var files = Directory.GetFiles(System.Web.HttpContext.Current.Server.MapPath($"~/stationImages/{station.StationId}/"), "*.jpg");
                station.Images = new List<string>();
                for (int i = 0; i < files.Count(); i++)
                {
                    station.Images.Add(files[i].Replace(AppDomain.CurrentDomain.BaseDirectory, "").Replace("\\", "/"));
                }

            }

            //增加场站各类充电枪的数量
            station.DCCount = station.DCIdleCount = station.ACSlowCount = station.ACSlowIdleCount = station.ACQuickCount = station.ACQuickIdleCount = 0;
            var stationGunState = redis.HashGet<StationGunState>(RedisKey.GunStateByStation, stationCode);
            if (stationGunState != null && stationGunState.gunState != null)
            {
                foreach (var item in stationGunState.gunState)
                {
                    switch (item.GunType)
                    {
                        case "直流":
                            station.DCCount = item.GunNum;
                            station.DCIdleCount = item.idleGun;
                            break;
                        case "单相交流":
                            station.ACSlowCount = item.GunNum;
                            station.ACSlowIdleCount = item.idleGun;
                            break;
                        case "三相交流":
                            station.ACQuickCount = item.GunNum;
                            station.ACQuickIdleCount = item.idleGun;
                            break;
                        default:
                            break;
                    }
                }
            }

            //维护场站的费用信息
            station.TodayElectricFeeList = GetTodayElectricityFee(station.StationId, DateTime.Now);
            station.TodayServiceFeeList = GetTodayServiceFeelist(station.StationId, DateTime.Now);
            station.CurrentElectricFee = GetCurrentElectricityFee(station.TodayElectricFeeList);
            station.CurrentServiceFee = GetCurrentServiceFee(station.TodayServiceFeeList);
            station.ChargeStrategy = GetChargeStrategy(station.StationId, DateTime.Now);
            station.ServiceFee = GetCurrentServiceFee(station.TodayServiceFeeList);//旧版本
            stations.Add(station);

            //维护场站评分和运营商联系方式
            List<string> stationidlist = new List<string>();
            try
            {
                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {
                    //如果有传用户标识进来，查询此用户收藏的场站
                    string openid = null;
                    if (!string.IsNullOrEmpty(usertoken))
                    {
                        openid = redis.HashGetString(RedisKey.Token, usertoken);
                    }
                    if (!string.IsNullOrEmpty(openid))
                    {
                        string sql = $"select station_id from lco_collect where user_id='{openid}'and station_id is not null AND station_id !=''";
                        stationidlist = SqlMapper.Query<string>(conn, sql).ToList();
                    }

                    string ratesql = $"SELECT isnull(avg(isnull(rate,5)/1.0),5) FROM lco_faults WHERE stationid = '{station.StationId}' AND Auditing = 0 and iscompany = 0  AND state = 0 AND isexist = 0";
                    string getphonesql = $"SELECT phone FROM lco_manufacturer WHERE cardInfo = '{station.CompanyID}'";
                    station.Phone = SqlMapper.Query<string>(conn, getphonesql).FirstOrDefault();
                    station.Score = SqlMapper.Query<double>(conn, ratesql).FirstOrDefault();
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetStationDetail数据库查询出错。stationcode={stationCode},usertoken={usertoken}.", ex);
            }

            //收藏的站中是否有此站
            if (stationidlist!=null && stationidlist.Contains(station.StationId))
            {
                station.IsCollect = 1;
            }

            //哪一个小程序
            List<string> stationofXM = redis.HashGet<List<string>>(RedisKey.StationOfApp, "xiaoma");
            if (stationofXM != null && stationofXM.Contains(stationCode))
            {
                station.app = EnumAppName.小马;
            }
            else
            {
                station.app = EnumAppName.领充;
            }

            //哪一种支付方式
            List<string> stationOfPay = redis.HashGet<List<string>>(RedisKey.StationOfApp, "UMSPay");
            if (stationOfPay != null && stationOfPay.Contains(stationCode))
            {
                station.payChannel = EnumPayChannel.银联商务;
            }
            else
            {
                station.payChannel = EnumPayChannel.微信;
            }
            

            return stations;

        }

        internal IEnumerable<LCO_Station> GetStationDetailByCode(string code, string usertoken)
        {
            if (string.IsNullOrEmpty(code))
            {
                return null;
            }
            string stationID = "";

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            
            if (redis.HashExists(RedisKey.Station, code))
            {
                //尝试判断传进来的是不是场站ID
                stationID = code;
            }
            else
            {
                //不是已知场站ID的话，认为是设备code
                stationID = WechatUtility.GetStationIDByDeviceCode(code);
            }

            return GetStationDetail(stationID, usertoken);
        }

        internal bool GeoAdd(string stationID, double longitude, double latitude)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            return redis.GeoAdd(RedisKey.GEO, stationID, longitude, latitude);
        }

        internal bool GeoRemove(string stationID, double longitude, double latitude)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            return redis.GeoRemove(RedisKey.GEO, stationID);
        }

        internal Dictionary<string, StackExchange.Redis.GeoPosition> GetNearStationSummary(double longitude, double latitude, double radius)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            return redis.GeoRadius(RedisKey.GEO, longitude, latitude, radius);
        }

        internal IEnumerable<LCO_Station> GetNearStation(double longitude, double latitude, double radius)
        {
            var result = new List<LCO_Station>();

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var nearStations = GetNearStationSummary(longitude, latitude, radius);

            foreach (var item in nearStations)
            {
                var station = redis.HashGet<LCO_Station>(RedisKey.Station, item.Key);
                if (station != null)
                {
                    station.TodayElectricFeeList = GetTodayElectricityFee(station.StationId, DateTime.Now);
                    station.TodayServiceFeeList = GetTodayServiceFeelist(station.StationId, DateTime.Now);
                    station.CurrentElectricFee = GetCurrentElectricityFee(station.TodayElectricFeeList);
                    station.CurrentServiceFee = GetCurrentServiceFee(station.TodayServiceFeeList);
                    station.ServiceFee = GetCurrentServiceFee(station.TodayServiceFeeList); //旧版本

                    result.Add(station);
                }
            }

            return result;
        }

        internal IEnumerable<LCO_Station> GetNearStation(string appname, string usertoken, double longitude, double latitude, double radius
            , int stationType, int chargeGunType, bool? isParkingFree, bool? isWorking, string companyID
            , EnumChargeStandardType chargeStandardType, EnumAuxiliaryPowerType auxiliaryPowerType
            , EnumStationOrderBy orderby = EnumStationOrderBy.距离最近)
        {
            var result = new List<LCO_Station>();
            string openid = null;
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            if (usertoken != null)
            {
                openid = redis.HashGetString(RedisKey.Token, usertoken);
            }
            var nearStations = GetNearStationSummary(longitude, latitude, radius);
            List<string> stationidlist = new List<string>();
            List<string> stationOfXM = redis.HashGet<List<string>>(RedisKey.StationOfApp, "xiaoma");
            using (IDbConnection conn = WechatUtility.CreateDBConnection())
            {
                foreach (var item in nearStations)
                {
                    var station = redis.HashGet<LCO_Station>(RedisKey.Station, item.Key);

                    bool isShow = IsStationShow(station, companyID, stationType, chargeGunType,
                        isParkingFree, isWorking, chargeStandardType, auxiliaryPowerType, null);
                    if (!isShow)
                    {
                        continue;
                    }

                    try
                    {
                        //最后订单时间
                        DateTime lastuseTime = SqlMapper.Query<DateTime>(conn, $"SELECT top 1 endtime FROM histroy_order_cost WHERE stationid = '{station.StationId}' ORDER BY endtime desc").FirstOrDefault();
                        if (lastuseTime != null)
                        {
                            station.LastSuccessTime = WechatUtility.format(lastuseTime);
                        }

                        //场站评分
                        string ratesql = $"SELECT isnull(avg(rate/1.00),5) FROM lco_faults WHERE stationid = '{station.StationId}' AND Auditing = 0 and iscompany = 0  AND state = 0 AND isexist = 0";
                        station.Score = SqlMapper.Query<double>(conn, ratesql).FirstOrDefault();

                        //场站图片
                        var files = GetStationImage(station.StationId);
                        if (files != null)
                        {
                            station.Images = new List<string>();
                            for (int i = 0; i < files.Count(); i++)
                            {
                                var a = files[i].Replace(AppDomain.CurrentDomain.BaseDirectory, "").Replace("\\", "/");
                                station.Images.Add(files[i].Replace(AppDomain.CurrentDomain.BaseDirectory, "").Replace("\\", "/"));
                            }
                        }

                        //各种类型数量统计
                        var stationGunState = redis.HashGet<StationGunState>(RedisKey.GunStateByStation, station.StationId);
                        station.DCCount = station.DCIdleCount = station.ACSlowCount = station.ACSlowIdleCount = station.ACQuickCount = station.ACQuickIdleCount = 0;
                        if (stationGunState != null && stationGunState.gunState != null)
                        {
                            foreach (var GunStateNum in stationGunState.gunState)
                            {
                                if (GunStateNum.GunType == "直流")
                                {
                                    station.DCCount = GunStateNum.GunNum;
                                    station.DCIdleCount = GunStateNum.idleGun;
                                }
                                else if (GunStateNum.GunType == "单相交流")
                                {
                                    station.ACSlowCount = GunStateNum.GunNum;
                                    station.ACSlowIdleCount = GunStateNum.idleGun;
                                }
                                else if (GunStateNum.GunType == "三相交流")
                                {
                                    station.ACQuickCount = GunStateNum.GunNum;
                                    station.ACQuickIdleCount = GunStateNum.idleGun;
                                }
                            }
                        }

                        station.TodayElectricFeeList = GetTodayElectricityFee(station.StationId, DateTime.Now);
                        station.TodayServiceFeeList = GetTodayServiceFeelist(station.StationId, DateTime.Now);
                        station.CurrentElectricFee = GetCurrentElectricityFee(station.TodayElectricFeeList);
                        station.CurrentServiceFee = GetCurrentServiceFee(station.TodayServiceFeeList);
                        station.ServiceFee = GetCurrentServiceFee(station.TodayServiceFeeList);//旧版本用
                        station.ChargeStrategy = GetChargeStrategy(station.StationId, DateTime.Now);
                    }
                    catch (Exception)
                    {
                        continue;
                    }

                    //判定是智电云联 还是 领充
                    if (stationOfXM != null && stationOfXM.Contains(station.StationId))
                    {
                        station.app = EnumAppName.小马;
                    }
                    else
                    {
                        station.app = EnumAppName.领充;
                    }
                    result.Add(station);
                }
                if (!string.IsNullOrEmpty(openid))
                {

                    string phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
                    if (openid.Length == 11)
                    {
                        phone = openid;
                    }
                    try
                    {
                        List<string> openidList = new List<string>();
                        string sql1 = $"SELECT openid FROM lco_wxcustomer WHERE phone = '{phone}'";
                        openidList = conn.Query<string>(sql1, conn).ToList();
                        string sql = $"select distinct station_id from lco_collect where user_id in (SELECT openid FROM lco_wxcustomer WHERE phone = '{phone}')and station_id is not null AND station_id !=''";
                        if (!openidList.Any())
                        {
                            sql = $"select distinct station_id from lco_collect where user_id ='{openid}' and station_id is not null AND station_id !=''";
                        }


                        stationidlist = SqlMapper.Query<string>(conn, sql).ToList();
                    }
                    catch (Exception e)
                    {
                        LogEvent.WebAPILoger.Error("查询收藏的错误！", e);
                    }

                    result.ForEach(p =>
                    {
                        if (stationidlist.Contains(p.StationId))
                        {
                            p.IsCollect = 1;
                        }
                    });
                }
                switch (orderby)
                {
                    case EnumStationOrderBy.距离最近:
                        break;
                    case EnumStationOrderBy.评分最高:
                        result = result.OrderByDescending(p => p.Score).ToList();
                        break;
                    case EnumStationOrderBy.空闲优先:
                        result = result.OrderByDescending(p => (p.DCIdleCount + p.ACQuickIdleCount + p.ACSlowIdleCount)).ToList();
                        break;
                    default:
                        break;
                }
            }
            return result;
        }

        /// <summary>
        /// 判断根据条件此场站是否需要在前端显示
        /// </summary>
        /// <param name="station"></param>
        /// <param name="companyID"></param>
        /// <param name="stationType"></param>
        /// <param name="chargeGunType"></param>
        /// <param name="isParkingFree"></param>
        /// <param name="isWorking"></param>
        /// <param name="chargeStandardType"></param>
        /// <param name="auxiliaryPowerType"></param>
        /// <returns></returns>
        internal bool IsStationShow(LCO_Station station,string companyID, int stationType, int chargeGunType, 
            bool? isParkingFree, bool? isWorking, EnumChargeStandardType chargeStandardType, EnumAuxiliaryPowerType auxiliaryPowerType,
            string keyword)
        {
            if (station == null)
            {
                return false;
            }

            if (station.StationState == EnumStationState.下架)
            {
                return false;
            }

            if (!string.IsNullOrEmpty(companyID) && station.CompanyID != companyID)
            {
                return false;
            }


            //场站类型是否符合，值按位从低到高依次代表为公共站、专用站、个人站
            //stationType==0 全不选就等于全选
            if (stationType > 0 && ((stationType & station.Stationtype) == 0))
            {
                return false;
            }

            //充电枪类型是否符合，值按位从低到高依次代表为交流慢充、交流快充、直流快充
            //chargeType==0 全不选就等于全选
            if (chargeGunType > 0 && ((chargeGunType & station.ChargeGunType) == 0))
            {
                return false;
            }

            //免费还是收费条件，为null的话不判断该条件
            if (isParkingFree.HasValue)
            {
                //ParkingFee为空字符代表免费;否则此字段显示停车费用
                if (isParkingFree.Value != (string.IsNullOrEmpty(station.ParkingFee) || station.ParkingFee.Contains("免费")))
                {
                    return false;
                }
            }

            //是否营业中条件
            if (isWorking.HasValue)
            {
                var startTime = DateTime.Parse(station.StartTime);
                var endTime = DateTime.Parse(station.EndTime);
                if (isWorking.Value)
                {
                    if (DateTime.Now < startTime || DateTime.Now > endTime)
                    {
                        return false;
                    }
                }
                else
                {
                    if (DateTime.Parse("00:00") != startTime || DateTime.Parse("23:59") != endTime)
                    {
                        return false;
                    }
                }
            }

            //是否符合充电标准接口 国标2011或者国标2015
            if (station.ChargeStandardType != EnumChargeStandardType.两者都有
                && (chargeStandardType != EnumChargeStandardType.两者都有 && chargeStandardType != station.ChargeStandardType))
            {
                return false;
            }

            //是否符合辅源类型 12V或者24V
            if (station.AuxiliaryPowerType != EnumAuxiliaryPowerType.两者都有
                && (auxiliaryPowerType != EnumAuxiliaryPowerType.两者都有 && auxiliaryPowerType != station.AuxiliaryPowerType))
            {
                return false;
            }

            //场站名是否包含关键字
            if (!string.IsNullOrEmpty(keyword) && !station.StationName.Contains(keyword))
            {
                return false;
            }

            return true;
        }

        internal IEnumerable<LCO_Station> GetStationByKeyword(double longitude, double latitude, string keyword, string appname, string companyid = null)
        {
            var result = new List<LCO_Station>();

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            List<string> stationIDs = new List<string>();

            if (string.IsNullOrEmpty(keyword))
            {
                double searchDefaultRadius = 200;
                var nearStations = GetNearStationSummary(longitude, latitude, searchDefaultRadius);
                stationIDs = nearStations.Keys.ToList();
            }
            else
            {
                stationIDs = redis.HashKeysString(RedisKey.Station);
            }
            
            foreach (var item in stationIDs)
            {
                var station = redis.HashGet<LCO_Station>(RedisKey.Station, item);
                //领充、小马地图及搜索显示筛选

                bool isShow = IsStationShow(station, companyid, 0, 0, null, null, 
                        EnumChargeStandardType.两者都有, EnumAuxiliaryPowerType.两者都有, keyword);
                if (!isShow)
                {
                    continue;
                }

                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {
                    //最近一次充电时间
                    DateTime lastuseTime = SqlMapper.Query<DateTime>(conn, $"SELECT top 1 endtime FROM histroy_order_cost WHERE stationid = '{station.StationId}' ORDER BY endtime desc").FirstOrDefault();
                    if (lastuseTime != null)
                    {
                        station.LastSuccessTime = WechatUtility.format(lastuseTime);
                    }

                    //场站评分
                    string ratesql = $"SELECT isnull(avg(rate/1.00),5) FROM lco_faults WHERE stationid = '{station.StationId}' AND Auditing = 0 and iscompany = 0  AND state = 0 AND isexist = 0";
                    station.Score = SqlMapper.Query<double>(conn, ratesql).FirstOrDefault();

                    //场站图片
                    var files = GetStationImage(station.StationId);
                    if (files != null)
                    {
                        station.Images = new List<string>();
                        for (int i = 0; i < files.Count(); i++)
                        {
                            station.Images.Add(files[i].Replace(AppDomain.CurrentDomain.BaseDirectory, "").Replace("\\", "/"));
                        }
                    }

                    //充电枪各类状态的数量
                    var stationGunState = redis.HashGet<StationGunState>(RedisKey.GunStateByStation, station.StationId);
                    station.DCCount = station.DCIdleCount = station.ACSlowCount = station.ACSlowIdleCount = station.ACQuickCount = station.ACQuickIdleCount = 0;
                    if (stationGunState != null && stationGunState.gunState != null)
                    {
                        foreach (var GunStateNum in stationGunState.gunState)
                        {
                            if (GunStateNum.GunType == "直流")
                            {
                                station.DCCount = GunStateNum.GunNum;
                                station.DCIdleCount = GunStateNum.idleGun;
                            }
                            else if (GunStateNum.GunType == "单相交流")
                            {
                                station.ACSlowCount = GunStateNum.GunNum;
                                station.ACSlowIdleCount = GunStateNum.idleGun;
                            }
                            else if (GunStateNum.GunType == "三相交流")
                            {
                                station.ACQuickCount = GunStateNum.GunNum;
                                station.ACQuickIdleCount = GunStateNum.idleGun;
                            }
                        }
                    }
                }

                //场站费用信息
                try
                {
                    station.TodayElectricFeeList = GetTodayElectricityFee(station.StationId, DateTime.Now);
                    station.TodayServiceFeeList = GetTodayServiceFeelist(station.StationId, DateTime.Now);
                    station.CurrentElectricFee = GetCurrentElectricityFee(station.TodayElectricFeeList);
                    station.CurrentServiceFee = GetCurrentServiceFee(station.TodayServiceFeeList);
                    station.ServiceFee = GetCurrentServiceFee(station.TodayServiceFeeList);//旧版本用
                    station.ChargeStrategy = GetChargeStrategy(station.StationId, DateTime.Now);
                }
                catch (Exception ex)
                {
                    LogEvent.WebAPILoger.Error($"GetStationByKeyword方法计算场站费用相关时出错。station={station.StationId}-{station.StationName}。",ex);
                }

                //运营商所属的服务商
                List<string> stationOfXM = redis.HashGet<List<string>>(RedisKey.StationOfApp, "xiaoma");
                if (stationOfXM != null && stationOfXM.Contains(station.StationId))
                {
                    station.app = EnumAppName.小马;
                }
                else
                {
                    station.app = EnumAppName.领充;
                }

                result.Add(station);

                ////每次最多返回20个结果，防止输入一个常见的字搜索
                //if (result.Count>=20)
                //{
                //    break;
                //}

            }
            return result;
        }

        internal IEnumerable<string> GetOtherInfo(string companyID = "")
        {
            List<string> infos = new List<string>();

            infos.Add(ConfigurationManager.AppSettings[companyID + "公众号"].ToString());
            infos.Add(ConfigurationManager.AppSettings[companyID + "客服电话"].ToString());
            infos.Add(ConfigurationManager.AppSettings[companyID + "联系邮箱"].ToString());
            infos.Add(ConfigurationManager.AppSettings[companyID + "商务合作"].ToString());
            infos.Add(ConfigurationManager.AppSettings[companyID + "公司网址"].ToString());
            infos.Add(ConfigurationManager.AppSettings[companyID + "公司名称"].ToString());

            return infos;
        }

        internal double? GetStationCurrentElectricity(string stationCode)
        {
            var station = GetStationDetail(stationCode);
            if (station == null || station.Count() == 0)
            {
                return null;
            }
            var electricity = station.First().CurrentElectricFee;
            return electricity;
        }

        internal List<Price> GetStationTodayElectricity(string stationCode)
        {
            var station = GetStationDetail(stationCode);
            if (station == null || station.Count() == 0)
            {
                return null;
            }
            var electricity = station.First().TodayElectricFeeList;
            return electricity;
        }

        internal List<Price> GetStationTodayServiceFee(string stationCode)
        {
            var station = GetStationDetail(stationCode);
            if (station == null || station.Count() == 0)
            {
                return null;
            }
            var serviceFee = station.First().TodayServiceFeeList;
            return serviceFee;
        }

        internal List<StrategyInfo> GetStationTodayChargeStrategy(string stationCode)
        {
            var station = GetStationDetail(stationCode);
            if (station == null || station.Count() == 0)
            {
                return null;
            }
            var chargeStrategy = station.First().ChargeStrategy;
            return chargeStrategy;
        }

        internal string[] GetStationImage(string stationID)
        {
            string[] files = null;
            if (Directory.Exists(System.Web.HttpContext.Current.Server.MapPath($"~/stationImages/{stationID}/")) == true)
            {
                files = Directory.GetFiles(System.Web.HttpContext.Current.Server.MapPath($"~/stationImages/{stationID}/"), "*.jpg");
            }
            return files;
        }
        #endregion

        #region Test

        public bool SaveVIP(string userToken, string nickName)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);

            var openid = redis.HashGetString(RedisKey.Token, userToken);
            LogEvent.WebAPILoger.Info($"userToken={userToken},openid={openid},nickName={nickName}");
            redis.HashSet(RedisKey.VIP, openid, nickName);

            return true;
        }

        #endregion

        #region 支付宝相关

        public string ALILogin(string code, string refreshToken)
        {
            string userToken = string.Empty;
            try
            {
                IAopClient client = new DefaultAopClient("https://openapi.alipay.com/gateway.do",
                    WxPayConfig.ALI_APP_ID, WxPayConfig.ALI_APP_PRIVATE_KEY, WxPayConfig.ALI_FORMAT, WxPayConfig.ALI_VERSION,
                    WxPayConfig.ALI_SIGN_TYPE, WxPayConfig.ALI_ALIPAY_PUBLIC_KEY, WxPayConfig.ALI_CHARSET, false);
                AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
                request.GrantType = string.IsNullOrEmpty(refreshToken) ? "authorization_code" : "refresh_token";
                request.Code = code;
                request.RefreshToken = refreshToken;
                AlipaySystemOauthTokenResponse response = client.Execute(request);
                if (!response.IsError)
                {
                    RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                    //以openid为Key，将openid、assess_key信息存入redis
                    userToken = response.AccessToken;
                    var openid = response.UserId;

                    var oldToken = redis.HashGetString(RedisKey.TokenReverse, openid);
                    if (!string.IsNullOrEmpty(oldToken))
                    {
                        redis.HashDelete(RedisKey.Token, oldToken);
                    }
                    redis.HashSet(RedisKey.TokenReverse, openid, userToken);
                    redis.HashSet(RedisKey.Token, userToken, openid);

                    //AlipayUserInfoShareRequest userInfoRequest = new AlipayUserInfoShareRequest();
                    //AlipayUserInfoShareResponse userInfoResponse = client.Execute(userInfoRequest, response.AccessToken);
                    //if (!userInfoResponse.IsError)
                    //{
                    //    RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                    //    //以openid为Key，将openid、assess_key信息存入redis
                    //    userToken = response.AccessToken;
                    //    var openid = userInfoResponse.UserId;
                    //    redis.HashSet(RedisKey.WXToken, userToken, openid);
                    //}
                    //else
                    //{
                    //    LogEvent.WebAPILoger.Error($"ChargeLogic - ALILogin - apiResult Error: response.Code={response.Code},response.Msg={response.Msg},response.SubCode={response.SubCode},response.SubMsg={response.SubMsg}");
                    //}
                }
                else
                {
                    LogEvent.WebAPILoger.Error($"ChargeLogic - ALILogin - apiResult Error: response.Code={response.Code},response.Msg={response.Msg},response.SubCode={response.SubCode},response.SubMsg={response.SubMsg}");
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("ChargeLogic - ALILogin", ex);
            }

            return userToken;
        }

        public string ALIMINAPay(WXMINAPayParameter param)
        {
            string orderString = "";
            try
            {
                IAopClient client = new DefaultAopClient("https://openapi.alipay.com/gateway.do",
                    WxPayConfig.ALI_APP_ID, WxPayConfig.ALI_APP_PRIVATE_KEY, WxPayConfig.ALI_FORMAT, WxPayConfig.ALI_VERSION,
                    WxPayConfig.ALI_SIGN_TYPE, WxPayConfig.ALI_ALIPAY_PUBLIC_KEY, WxPayConfig.ALI_CHARSET, false);

                //实例化具体API对应的request类,类名称和接口名称对应,当前调用接口名称如：alipay.trade.app.pay
                AlipayTradeAppPayRequest request = new AlipayTradeAppPayRequest();
                //SDK已经封装掉了公共参数，这里只需要传入业务参数。以下方法为sdk的model入参方式(model和biz_content同时存在的情况下取biz_content)。
                AlipayTradeAppPayModel model = new AlipayTradeAppPayModel();
                model.Body = string.IsNullOrEmpty(ChargeLogic.WXBody) ? "充电费用" : ChargeLogic.WXBody;
                model.Subject = "充电费用";
                model.TotalAmount = (param.fee / 100.0).ToString("F2");
                model.ProductCode = "QUICK_MSECURITY_PAY";
                model.OutTradeNo = string.Format("{0}{1}", DateTime.Now.ToString("yyMMddHHmmssfff"), new Random().Next(100, 999));
                model.TimeoutExpress = "5m";
                model.PassbackParams = param.deviceCode;
                ExtendParams extendParams = new ExtendParams();
                extendParams.SysServiceProviderId = "2088821552858213";//PID
                model.ExtendParams = extendParams;
                request.SetBizModel(model);
                request.SetNotifyUrl(WxPayConfig.ALI_NOTIFY_URL);
                //这里和普通的接口调用不同，使用的是sdkExecute
                AlipayTradeAppPayResponse response = client.SdkExecute(request);

                //response.Body就是orderString 可以直接给客户端请求，无需再做处理。
                orderString = response.Body;
                //LogEvent.WebAPILoger.Debug($"orderString={orderString}");
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"ALIMINAPay Error param ={param.ToString()}");
                LogEvent.WebAPILoger.Error("ALIMINAPay Error=", ex);
            }

            return orderString;
        }

        public string ALIMINAPayNotify()
        {
            IDictionary<string, string> sArray = new Dictionary<string, string>();
            //Load Form variables into NameValueCollection variable.
            NameValueCollection coll = System.Web.HttpContext.Current.Request.Form;
            // Get names of all forms into a string array.
            String[] requestItem = coll.AllKeys;
            for (var i = 0; i < requestItem.Length; i++)
            {
                sArray.Add(requestItem[i], System.Web.HttpContext.Current.Request.Form[requestItem[i]]);
                LogEvent.WebAPILoger.Info($"ALIMINAPayNotify Request.Form key={requestItem[i]},value={sArray[requestItem[i]]}");
            }

            bool RSACheckResult = AlipaySignature.RSACheckV1(sArray, WxPayConfig.ALI_ALIPAY_PUBLIC_KEY, WxPayConfig.ALI_CHARSET, WxPayConfig.ALI_SIGN_TYPE, false);

            if (RSACheckResult && sArray["trade_status"] == "TRADE_SUCCESS")
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                string openid = sArray["buyer_id"];
                int total_fee = (int)(decimal.Parse(sArray["total_amount"]) * 100);
                string nickName = sArray["buyer_logon_id"];
                string deviceCode = sArray["passback_params"];
                string stationCode = deviceCode.Split('|')[0];
                string transaction_id = sArray["trade_no"];
                string out_trade_no = sArray["out_trade_no"];
                long time_end = long.Parse(sArray["gmt_payment"].Replace("-", "").Replace(" ", "").Replace(":", ""));
                string companyID = redis.HashGet<LCO_Station>(RedisKey.Station, stationCode).CompanyID;

                if (new PayLogic().CreateRedisOrderFuwushang("", openid, total_fee, nickName, deviceCode, stationCode, transaction_id, out_trade_no, time_end, EnumPaymentType.银商支付宝, companyID))
                {
                    return "success";
                }
            }
            else
            {
                LogEvent.WebAPILoger.Error($"{DateTime.Now} RSACheckV1 Result = {RSACheckResult}");
                LogEvent.WebAPILoger.Error($"{DateTime.Now} ALIMINAPayNotify sArray[\"trade_status\"] = {sArray["trade_status"]}");
            }

            return "";
        }

        public string ALIRefund(WXRefund refundParameter)
        {
            string resultString = "";
            try
            {
                IAopClient client = new DefaultAopClient("https://openapi.alipay.com/gateway.do",
                    WxPayConfig.ALI_APP_ID, WxPayConfig.ALI_APP_PRIVATE_KEY, WxPayConfig.ALI_FORMAT, WxPayConfig.ALI_VERSION,
                    WxPayConfig.ALI_SIGN_TYPE, WxPayConfig.ALI_ALIPAY_PUBLIC_KEY, WxPayConfig.ALI_CHARSET, false);

                AlipayTradeRefundRequest refundRequest = new AlipayTradeRefundRequest();

                AlipayTradeRefundModel refundModel = new AlipayTradeRefundModel();
                refundModel.OperatorId = "system";
                refundModel.OutRequestNo = refundParameter.Out_trade_no;
                refundModel.OutTradeNo = refundParameter.Out_trade_no;
                refundModel.RefundAmount = (refundParameter.Refund_fee / 100.0).ToString();
                refundModel.RefundReason = "充电剩余费用";
                //refundModel.StoreId = "";
                //refundModel.TerminalId = "";
                refundModel.TradeNo = refundParameter.Transaction_id;
                refundRequest.SetBizModel(refundModel);

                AlipayTradeRefundResponse refundResponse = client.Execute(refundRequest);

                resultString = refundResponse.Body;
                LogEvent.WebAPILoger.Debug($"ALIRefund resultString={resultString}");
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("ALIRefund Error=", ex);
            }

            return resultString;
        }

        #endregion

        #region 威富通相关

        //public string WFTMINAPay(WXMINAPayParameter param)
        //{
        //    var openid = GetOpenIDByUserToken(param.userToken);

        //    var parameters = GetPayInfo(openid, param);

        //    return parameters;
        //}

        //private string GetPayInfo(string openID, WXMINAPayParameter parameter)
        //{
        //    string openid = WechatUtility.GetOpenIDByUserToken(parameter.userToken);
        //    string result = string.Empty;
        //    var reqHandler = new RequestHandler(null);
        //    var cfg = new Dictionary<string, string>();
        //    var pay = new PayHttpClient();
        //    var resHandler = new ClientResponseHandler();
        //    LCO_Station station = WechatUtility.GetStation(parameter.deviceCode);
        //    //TODO:小程序需要传参增加一个本身的appid参数
        //    //param.companyID = station.CompanyID;//运营商ID以device对应的station中的companyID为准，前端传给的不用
        //    WXSubMCHInfo wxSubMCHInfo = new PayLogic().GetSubAPPIDandMCHID(station.CompanyID);

        //    //加载配置数据
        //    cfg = Utils.loadCfg();
        //    //初始化数据  
        //    reqHandler.setGateUrl(WxPayConfig.WFTreq_url);
        //    //reqHandler.setKey(cfg["key"].ToString());
        //    reqHandler.setParameter("out_trade_no", Utils.Nmrandom());//商户订单号
        //    reqHandler.setParameter("body", string.IsNullOrEmpty(ChargeLogic.WXBody) ? "西安特锐德-充电费用" : ChargeLogic.WXBody);//商品描述
        //    reqHandler.setParameter("attach", parameter.AppName + "-" + parameter.companyID + "-" + parameter.phone);//附加信息
        //    reqHandler.setParameter("total_fee", parameter.fee.ToString());//总金额
        //    reqHandler.setParameter("mch_create_ip", "127.0.0.1");//终端IP
        //    reqHandler.setParameter("time_start", DateTime.Now.ToString("yyyyMMddHHmmss")); //订单生成时间
        //    reqHandler.setParameter("time_expire", DateTime.Now.AddMinutes(10).ToString("yyyyMMddHHmmss"));//订单超时时间
        //    reqHandler.setParameter("mch_id", cfg["mch_id"].ToString());//必填项，商户号，由平台分配
        //    reqHandler.setParameter("version", cfg["version"].ToString());//接口版本号
        //    reqHandler.setParameter("notify_url", WxPayConfig.WFTnotify_url);//通知地址，必填项，接收平台通知的URL，需给绝对路径，255字符内;此URL要保证外网能访问   

        //    if (parameter.paymenttype == EnumPaymentType.威富通微信)
        //    {
        //        reqHandler.setParameter("service", "unified.trade.pay");//接口类型：pay.weixin.jspay
        //        reqHandler.setParameter("sub_appid", "wx2a5538052969956e");
        //    }

        //    else if (parameter.paymenttype == EnumPaymentType.威富通云闪付)
        //    {
        //        reqHandler.setParameter("service", "pay.unionpay.raw.app");
        //    }
        //    else
        //    {
        //        reqHandler.setParameter("service", "pay.weixin.jspay");//接口类型：pay.weixin.jspay
        //        reqHandler.setParameter("is_raw", "1");//原生JS值
        //        reqHandler.setParameter("is_minipg", "1");//值为1，表示小程序支付；不传或值不为1，表示公众账号内支付
        //        reqHandler.setParameter("sub_openid", "");//测试账号不传值,此处默认给空值。正式账号必须传openid值，获取openid值指导文档地址：http://www.cnblogs.com/txw1958/p/weixin76-user-info.html
        //    }
        //    reqHandler.setParameter("mch_id", wxSubMCHInfo.MCHID);//必填项，商户号，由平台分配
        //    reqHandler.setParameter("version", WxPayConfig.WFTversion);//接口版本号
        //    reqHandler.setParameter("notify_url", WxPayConfig.WFTnotify_url);//通知地址，必填项，接收平台通知的URL，需给绝对路径，255字符内;此URL要保证外网能访问   
        //    reqHandler.setParameter("nonce_str", Utils.random());//随机字符串，必填项，不长于 32 位
        //    reqHandler.setParameter("charset", "UTF-8");//字符集

        //    reqHandler.setParameter("sign_type", "RSA_1_256");//签名方式
        //    reqHandler.setParameter("groupno", "131550003388");

        //    reqHandler.setParameter("device_info", parameter.deviceCode);//终端设备号

        //    //reqHandler.setParameter("sub_appid", WxPayConfig.WXAAPPID);//当发起公众号支付时，值是微信公众平台基本配置中的AppID(应用ID)；当发起小程序支付时，值是对应小程序的AppID
        //    reqHandler.setParameter("callback_url", "https://www.swiftpass.cn");//前台地址  交易完成后跳转的 URL，需给绝对路径，255字 符 内 格 式如:http://wap.tenpay.com/callback.asp
        //    reqHandler.setParameter("goods_tag", "");//商品标记

        //    //app 微信支付

        //    if (parameter.paymenttype == EnumPaymentType.威富通微信)
        //    {
        //        reqHandler.setParameter("service", "unified.trade.pay");//接口类型：unified.trade.pay


        //        reqHandler.setParameter("mch_id", cfg["mch_id"].ToString());
        //    }
        //    //app 云闪付支付
        //    else if (parameter.paymenttype == EnumPaymentType.威富通云闪付)
        //    {
        //        reqHandler.setParameter("service", "pay.unionpay.raw.app");
        //    }

        //    //小程序支付
        //    else
        //    {
        //        reqHandler.setParameter("service", "pay.weixin.jspay");//接口类型：pay.weixin.jspay
        //        reqHandler.setParameter("is_raw", "1");//原生JS值
        //        ;
        //        reqHandler.setParameter("is_minipg", "1");//值为1，表示小程序支付；不传或值不为1，表示公众账号内支付
        //        reqHandler.setParameter("sub_openid", "oR1oA0fp2EzeuG_ruG3VwhioJxt8");//测试账号不传值,此处默认给空值。正式账号必须传openid值，获取openid值指导文档地址：http://www.cnblogs.com/txw1958/p/weixin76-user-info.html
        //    }

        //    reqHandler.createSign();//创建签名

        //    //以上参数进行签名
        //    string data = Utils.toXml(reqHandler.getAllParameters());//生成XML报文
        //    Dictionary<string, string> reqContent = new Dictionary<string, string>();
        //    reqContent.Add("url", reqHandler.getGateUrl());
        //    reqContent.Add("data", data);
        //    pay.setReqContent(reqContent);

        //    if (pay.call())
        //    {
        //        resHandler.setContent(pay.getResContent());
        //        // resHandler.setKey(cfg["mchPrivateKey"].ToString());
        //        Hashtable param = resHandler.getAllParameters();
        //        //威富通APP
        //        if (parameter.paymenttype == EnumPaymentType.威富通微信)

        //        {

        //            if (int.Parse(param["status"].ToString()) == 0 && resHandler.isTenpaySign()) { result = param["token_id"].ToString(); }
        //            else
        //            {
        //                LogEvent.WebAPILoger.Error($"威富通APP下单错误，错误代码err_code={param["err_code"]}，错误信息err_msg={param["err_msg"]}");
        //                LogEvent.WebAPILoger.Error($"威富通APP下单错误，错误代码status={param["status"]}，错误信息message={param["message"]}");
        //            }
        //        }
        //        else if (parameter.paymenttype == EnumPaymentType.威富通云闪付)
        //        {
        //            if (int.Parse(param["status"].ToString()) == 0 && resHandler.isTenpaySign()) { result = param["tn"].ToString(); }
        //            else
        //            {
        //                LogEvent.WebAPILoger.Error($"威富通云闪付下单错误，错误代码err_code={param["err_code"]}，错误信息err_msg={param["err_msg"]}");
        //                LogEvent.WebAPILoger.Error($"威富通云闪付下单错误，错误代码status={param["status"]}，错误信息message={param["message"]}");
        //            }
        //        }
        //        else//威富通微信小程序
        //        {
        //            if (resHandler.isTenpaySign())
        //            {
        //                //当返回状态与业务结果都为0时才返回，其它结果请查看接口文档
        //                if (int.Parse(param["status"].ToString()) == 0 && int.Parse(param["result_code"].ToString()) == 0)
        //                {
        //                    result = param["pay_info"].ToString();
        //                }
        //                else
        //                {
        //                    LogEvent.WebAPILoger.Error($"威富通小程序下单错误，错误代码err_code={param["err_code"]}，错误信息err_msg={param["err_msg"]}");
        //                    LogEvent.WebAPILoger.Error($"威富通小程序下单错误，错误代码status={param["status"]}，错误信息message={param["message"]}");
        //                }
        //            }
        //            else
        //            {
        //                LogEvent.WebAPILoger.Error($"威富通小程序下单错误，错误代码err_code={param["err_code"]}，错误信息err_msg={param["err_msg"]}");
        //                LogEvent.WebAPILoger.Error($"威富通小程序下单错误，错误代码status={param["status"]}，错误信息message={param["message"]}");

        //            }
        //        }
        //    }
        //    else
        //    {
        //        LogEvent.WebAPILoger.Error($"威富通下单错误，错误代码getResponseCode={pay.getResponseCode()}，错误信息getErrInfo={pay.getErrInfo()}");
        //    }

        //    return result;
        //}

        //private bool ISSuccessOrder(string transaction_id, string out_trade_no, string mch_id)
        //{
        //    bool result = false;
        //    ClientResponseHandler resHandler = new ClientResponseHandler();
        //    PayHttpClient pay = new PayHttpClient();
        //    RequestHandler reqHandler = null;
        //    Dictionary<string, string> cfg = new Dictionary<string, string>();

        //    try
        //    {
        //        reqHandler = new RequestHandler(null);
        //        //加载配置数据
        //        cfg = Utils.loadCfg(); ;
        //        //初始化数据 
        //        reqHandler.setGateUrl(cfg["req_url"].ToString());
        //        reqHandler.setKey(cfg["key"].ToString());
        //        reqHandler.setParameter("out_trade_no", out_trade_no);//商户订单号
        //        reqHandler.setParameter("transaction_id", transaction_id);//平台订单号     

        //        reqHandler.setParameter("service", "unified.trade.query");//接口 unified.trade.refund 
        //        reqHandler.setParameter("mch_id", mch_id);//必填项，商户号，由平台分配
        //        reqHandler.setParameter("version", cfg["version"].ToString());//接口版本号
        //        reqHandler.setParameter("nonce_str", Utils.random());//随机字符串，必填项，不长于 32 位
        //        reqHandler.createSign();//创建签名
        //                                //以上参数进行签名
        //        string data = Utils.toXml(reqHandler.getAllParameters());//生成XML报文
        //        Dictionary<string, string> reqContent = new Dictionary<string, string>();
        //        reqContent.Add("url", reqHandler.getGateUrl());
        //        reqContent.Add("data", data);

        //        pay.setReqContent(reqContent);

        //        if (pay.call())
        //        {
        //            resHandler.setContent(pay.getResContent());
        //            resHandler.setKey(cfg["key"].ToString());
        //            Hashtable param = resHandler.getAllParameters();
        //            if (resHandler.isTenpaySign())
        //            {
        //                //当返回状态与业务结果都为0时才返回结果，其它结果请查看接口文档
        //                if (int.Parse(param["status"].ToString()) == 0 && int.Parse(param["result_code"].ToString()) == 0 && param["trade_state"].ToString() == "SUCCESS")
        //                {
        //                    result = true;
        //                }
        //                else
        //                {
        //                    LogEvent.WebAPILoger.Error($"威富通订单异常原因：message = {param["message"]}");
        //                }
        //            }
        //            else
        //            {
        //                LogEvent.WebAPILoger.Error($"威富通查询订单错误，错误代码status={param["status"]}，错误信息message={param["message"]}");
        //            }
        //        }
        //        else
        //        {
        //            LogEvent.WebAPILoger.Error($"威富通查询订单错误，错误代码getResponseCode={pay.getResponseCode()}，错误信息getErrInfo={pay.getErrInfo()}");
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        LogEvent.WebAPILoger.Error("威富通退款异常 Error=", ex);
        //    }
        //    return result;
        //}

        //public string WFTMINAPayNotify()
        //{
        //    ClientResponseHandler resHandler = new ClientResponseHandler();
        //    Dictionary<string, string> cfg = new Dictionary<string, string>();

        //    //加载配置数据
        //    cfg = Utils.loadCfg();
        //    //初始化数据
        //    using (StreamReader sr = new StreamReader(System.Web.HttpContext.Current.Request.InputStream))
        //    {
        //        resHandler.setContent(sr.ReadToEnd());
        //        resHandler.setKey(cfg["key"]);

        //        Hashtable resParam = resHandler.getAllParameters();
        //        bool result = ISSuccessOrder(resParam["transaction_id"].ToString(), resParam["out_trade_no"].ToString(), resParam["mch_id"].ToString());
        //        if (resHandler.isTenpaySign() && result == true)
        //        {
        //            if (int.Parse(resParam["status"].ToString()) == 0 && int.Parse(resParam["result_code"].ToString()) == 0)
        //            {
        //                Utils.writeFile("接口回调", resParam); //通知返回参数写入result.txt文本文件。
        //                                                   //此处可以在添加相关处理业务，校验通知参数中的商户订单号out_trade_no和金额total_fee是否和商户业务系统的单号和金额是否一致，一致后方可更新数据库表中的记录。  
        //                LogEvent.WebAPILoger.Info("威富通回调正常进入。");

        //                var openid = resParam["openid"].ToString();
        //                var total_fee = int.Parse(resParam["total_fee"].ToString());
        //                var nickName = resParam["attach"].ToString();
        //                var deviceCode = resParam["device_info"].ToString();
        //                var stationCode = deviceCode.Split('|')[0];
        //                var transaction_id = resParam["transaction_id"].ToString();
        //                var out_trade_no = resParam["out_trade_no"].ToString();
        //                var time_end = long.Parse(resParam["time_end"].ToString());

        //                if (CreateRedisOrder(openid, total_fee, nickName, deviceCode, stationCode, transaction_id, out_trade_no, time_end, EnumPaymentType.威富通微信))
        //                {
        //                    return "success";
        //                }
        //            }
        //            else
        //            {
        //                LogEvent.WebAPILoger.Error($"威富通回调错误，返回状态或返回code不正确。status={resParam["status"].ToString()}，result_code={resParam["result_code"].ToString()}");
        //            }
        //        }
        //        else
        //        {
        //            LogEvent.WebAPILoger.Error($"威富通回调错误，签名不通过。");
        //        }
        //    }

        //    return "";
        //}

        //public void SetWFTRefund(WXRefund param)
        //{
        //    RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
        //    redis.ListLeftPush(RedisKey.WFTRefund, param);
        //}

        //public string WFTRefund(WXRefund refundParameter)
        //{
        //    string refundResponse = string.Empty;
        //    ClientResponseHandler resHandler = new ClientResponseHandler();
        //    PayHttpClient pay = new PayHttpClient();
        //    RequestHandler reqHandler = null;
        //    Dictionary<string, string> cfg = new Dictionary<string, string>();

        //    try
        //    {
        //        reqHandler = new RequestHandler(null);
        //        //加载配置数据
        //        cfg = Utils.loadCfg(); ;
        //        //初始化数据 
        //        reqHandler.setGateUrl(cfg["req_url"].ToString());
        //        reqHandler.setKey(cfg["key"].ToString());
        //        reqHandler.setParameter("out_trade_no", refundParameter.Out_trade_no);//商户订单号
        //        reqHandler.setParameter("transaction_id", refundParameter.Transaction_id);//平台订单号     
        //        reqHandler.setParameter("out_refund_no", refundParameter.Out_trade_no);//商户退款单号
        //        reqHandler.setParameter("total_fee", refundParameter.Total_fee.ToString());//总金额
        //        reqHandler.setParameter("refund_fee", refundParameter.Refund_fee.ToString());//退款金额
        //        reqHandler.setParameter("refund_channel", "ORIGINAL");//退款渠道 ORIGINAL-原路退款，默认

        //        reqHandler.setParameter("service", "unified.trade.refund");//接口 unified.trade.refund 
        //        reqHandler.setParameter("mch_id", cfg["mch_id"].ToString());//必填项，商户号，由平台分配
        //        reqHandler.setParameter("version", cfg["version"].ToString());//接口版本号
        //        reqHandler.setParameter("op_user_id", cfg["mch_id"]);//必填项，操作员帐号,默认为商户号
        //        reqHandler.setParameter("nonce_str", Utils.random());//随机字符串，必填项，不长于 32 位
        //        reqHandler.createSign();//创建签名
        //                                //以上参数进行签名
        //        string data = Utils.toXml(reqHandler.getAllParameters());//生成XML报文
        //        Dictionary<string, string> reqContent = new Dictionary<string, string>();
        //        reqContent.Add("url", reqHandler.getGateUrl());
        //        reqContent.Add("data", data);

        //        pay.setReqContent(reqContent);

        //        if (pay.call())
        //        {
        //            resHandler.setContent(pay.getResContent());
        //            resHandler.setKey(cfg["key"].ToString());
        //            Hashtable param = resHandler.getAllParameters();
        //            if (resHandler.isTenpaySign())
        //            {
        //                refundResponse = HashtableToJson(param);
        //            }
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        LogEvent.WebAPILoger.Error("威富通退款异常 Error=", ex);
        //    }

        //    return refundResponse;
        //}

        //public static string HashtableToJson(Hashtable hr, int readcount = 0)

        //{
        //    string json = "{";
        //    foreach (DictionaryEntry row in hr)
        //    {
        //        try
        //        {
        //            string key = "\"" + row.Key + "\":";
        //            if (row.Value is Hashtable)
        //            {
        //                Hashtable t = (Hashtable)row.Value;
        //                if (t.Count > 0)
        //                {
        //                    json += key + HashtableToJson(t, readcount++) + ",";
        //                }
        //                else { json += key + "{},"; }
        //            }
        //            else
        //            {
        //                string value = "\"" + row.Value.ToString() + "\",";
        //                json += key + value;
        //            }
        //        }
        //        catch { }
        //    }

        //    json = json + "}";
        //    return json;
        //}
        #endregion

        #region APP相关

        public APPVersion CheckAPPVersion(int versionCode)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            APPVersion appVersion = new APPVersion();
            appVersion = redis.HashGet<APPVersion>(RedisKey.AppInfo, "9999");

            //大版本一致，小版本不一致建议更新；大版本不一致强制更新
            if (versionCode >= appVersion.versionCode)
            {
                appVersion.update = (int)EnumAPPUpdate.无需更新;
            }
            else if (versionCode / 100 == appVersion.versionCode / 100)
            {
                appVersion.update = (int)EnumAPPUpdate.建议更新;
            }
            else
            {
                appVersion.update = (int)EnumAPPUpdate.强制更新;
            }
            return appVersion;
        }

        public int SendPIN(PINInfo requestInfo)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var isVIP = redis.HashExists(RedisKey.VIP, requestInfo.PhoneNum);
            if (isVIP)
            {
                return 1;
            }

            var originalText = requestInfo.PhoneNum + "wutangpeifang" + requestInfo.RequestTime;
            var md5 = MD5(originalText);
            if (!requestInfo.MD5.ToUpper().Equals(md5))
            {
                // md5 check is wrong
                return -3;
            }

            var pinInfo = redis.HashGet<PINInfo>(RedisKey.PIN, requestInfo.PhoneNum);
            var now = DateTime.Now;
            if (pinInfo != null)
            {
                if (now.Date.Subtract(pinInfo.LastSendTime.Value.Date).Days == 0 && pinInfo.TodaySendCount > 5)
                {
                    // sent more than maximum
                    return -1;
                }

                if (now.Subtract(pinInfo.LastSendTime.Value).TotalSeconds <= 60)
                {
                    // sending interval is too short
                    return -2;
                }
            }
            else
            {
                pinInfo = requestInfo;
            }

            pinInfo.PIN = new Random().Next(1000, 9999).ToString();
            if (pinInfo.LastSendTime.HasValue && now.Date.Subtract(pinInfo.LastSendTime.Value.Date).Days == 0)
            {
                pinInfo.TodaySendCount += 1;
            }
            else
            {
                pinInfo.TodaySendCount = 1;
            }
            pinInfo.LastSendTime = now;


            redis.HashSet<PINInfo>(RedisKey.PIN, pinInfo.PhoneNum, pinInfo);

            //var sms = new SMS_5C();
            //var smsContent = $"您的验证码是{pinInfo.PIN}。请在页面中提交验证码完成验证。";
            //sms.Send(new List<string> { pinInfo.PhoneNum }, new List<string> { smsContent });
            int sendResult = SendPIN_ALiSMS(pinInfo);


            return 1;
        }

        private int SendPIN_ALiSMS(PINInfo requestInfo)
        {
            int result = 0;
            try
            {
                DateTime dtNow = DateTime.Parse(DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.fff"));
                long timetick = GetTicks(dtNow);
                string slat = "krb9wmr1xxl9jcfhz2gh30v3xqm05amv";

                Dictionary<string, dynamic> list = new Dictionary<string, dynamic>();
                list.Add("phoneNumbers", requestInfo.PhoneNum);
                list.Add("signName", SendSMSCompany);
                list.Add("templateCode", TemplateCode);
                list.Add("templateParam", "{\"code\":\"" + requestInfo.PIN + "\"}");
                //转换成json
                string paramJson = JsonConvert.SerializeObject(list);
                string param = AESUtil.Encrypt(paramJson, "xRhjcqtbxU38LpUS", "hbsFjWAvTDxLXMtm");
                //
                string data = string.Format("{0}{1}{2}", param, slat, timetick);
                string sign = MD5Hash(data);

                Dictionary<string, dynamic> sendData = new Dictionary<string, dynamic>();
                sendData.Add("timeStamp", timetick);
                sendData.Add("data", param);
                sendData.Add("sign", sign.ToUpper());
                //转换成json
                var json = JsonConvert.SerializeObject(sendData);
                string apiResult = "";
                using (var httpClient = new HttpClient())
                {
                    //放入httpcontent里面
                    HttpContent httpContent = new StringContent(json);
                    httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                    //当参数传递
                    HttpResponseMessage httpResponseMessage = httpClient.PostAsync(SendSMSApi, httpContent).Result;
                    if (httpResponseMessage.StatusCode == System.Net.HttpStatusCode.OK ||
                        httpResponseMessage.StatusCode == System.Net.HttpStatusCode.NoContent)
                    {
                        LogEvent.WebAPILoger.Info($"{requestInfo.PhoneNum}账户验证码发送成功。");
                        result = 1;
                    }
                    else
                    {
                        apiResult = httpResponseMessage.Content.ReadAsStringAsync().Result;
                        LogEvent.WebAPILoger.Error($"{requestInfo.PhoneNum}账户验证码发送失败。");
                    }
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"{requestInfo.PhoneNum}账户验证码发送异常", ex);
            }
            return result;
        }

        private long GetTicks(DateTime dt)
        {
            // epoch 就是 1970-01-01 00:00:00.000 这个时刻
            //Java 是以 UTC 为基准的，.NET 中与其对应的是 DateTime.UtcNow 而非 DateTime.Now
            //return Decimal.ToInt64(Decimal.Divide(DateTime.Now.Ticks - new DateTime(1970, 1, 1, 8, 0, 0).Ticks, 10000));
            return Decimal.ToInt64(Decimal.Divide(dt.Ticks - 621355968000000000, 10000));
        }
        private string MD5Hash(string input)
        {
            using (var md5 = System.Security.Cryptography.MD5.Create())
            {
                var result = md5.ComputeHash(Encoding.ASCII.GetBytes(input));
                var strResult = BitConverter.ToString(result);
                return strResult.Replace("-", "");
            }
        }
        internal JObject CheckPIN(KeyValuePair<string, string> phoneNumCode)
        {
            JObject result = null;

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var pinInfo = redis.HashGet<PINInfo>(RedisKey.PIN, phoneNumCode.Key);

            if (pinInfo != null && !string.IsNullOrEmpty(pinInfo.PIN)
                && pinInfo.PIN.Equals(phoneNumCode.Value)
                && DateTime.Now.Subtract(pinInfo.LastSendTime.Value).TotalSeconds < 600)
            {
                // 验证通过，则把此手机号记录到数据库
                bool dbResult = InsertAppUsertoDB(phoneNumCode.Key, phoneNumCode.Key);
                if (dbResult)
                {
                    // 在Redis中记录此手机号
                    var userToken = Guid.NewGuid().ToString("N");
                    SaveTokenToRedis(userToken, phoneNumCode.Key);

                    // 从redis中获取用户
                    var app_user = redis.HashGet<APP_User>(RedisKey.APPUser, phoneNumCode.Key);
                    // Redis中不存在，则从数据库获取一次
                    if (app_user == null)
                    {
                        app_user = GetAPPUserFromDB(phoneNumCode.Key);
                    }
                    app_user.Timestamp = DateTime.Now;
                    app_user.DeviceID = Guid.NewGuid().ToString("N");
                    redis.HashSet<APP_User>(RedisKey.APPUser, pinInfo.PhoneNum, app_user);
                    UserLogic userlogic = new UserLogic();
                    userlogic.SaveUserAndPhone(pinInfo.PhoneNum, pinInfo.PhoneNum, pinInfo.PhoneNum);

                    result = new JObject();
                    result.Add("UserToken", userToken);
                    result.Add("DeviceID", app_user.DeviceID);
                }
            }
            return result;
        }

        public JObject APPLogin(APP_Login login)
        {
            JObject result = null;

            var originalText = login.DeviceID + "冰力十足";
            var md5 = MD5(originalText);
            if (!login.MD5.Equals(md5))
            {
                return result;
            }

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var app_user = redis.HashGet<APP_User>(RedisKey.APPUser, login.PhoneNum);
            // Redis中不存在，则认为未校验过短信验证码
            if (app_user == null)
            {
                return result;
            }
            // 这次登录的设备如果和上次此用户登录的不一样，则需要重新验证短信
            if (app_user.DeviceID != login.DeviceID)
            {
                return result;
            }

            // 在Redis中记录此手机号
            var userToken = Guid.NewGuid().ToString("N");

            SaveTokenToRedis(userToken, login.PhoneNum);



            // 更新用户登录时间
            app_user.Timestamp = DateTime.Now;
            //app_user.DeviceID = Guid.NewGuid().ToString("N");
            redis.HashSet<APP_User>(RedisKey.APPUser, app_user.loginid, app_user);

            result = new JObject();
            result.Add("UserToken", userToken);
            result.Add("DeviceID", login.DeviceID);

            return result;
        }

        private bool InsertAppUsertoDB(string loginID, string nickName)
        {
            bool result = false;

            DynamicParameters param = new DynamicParameters();
            param.Add("@loginid", loginID);
            param.Add("@nickname", nickName);
            //param.Add("@profile", profile);
            //param.Add("@sex", sex);
            //param.Add("@birthday", birthday);
            //param.Add("@notes", notes);

            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var spName = "APP_CreateUser";
                int affectedRowNum = connection.Execute(spName, param, null, null, CommandType.StoredProcedure);
                result = true;
            }

            return result;
        }

        private string MD5(string data)
        {
            var hash = System.Security.Cryptography.MD5.Create();

            var bytes = Encoding.UTF8.GetBytes(data);
            var md5Val = hash.ComputeHash(bytes);
            //把二进制转化为大写的十六进制
            StringBuilder result = new StringBuilder();
            foreach (var c in md5Val)
            {
                result.Append(c.ToString("X2"));
            }

            return result.ToString().ToUpper();
        }

        public APP_User GetAPPUser(string userToken)
        {
            if (string.IsNullOrEmpty(userToken))
            {
                return null;
            }
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var openid = GetOpenIDByUserToken(userToken);
            var app_user = redis.HashGet<APP_User>(RedisKey.APPUser, openid);

            if (string.IsNullOrEmpty(openid))
            {
                return null;
            }
            try
            {
                app_user.profile = GetAPPUserProfile(openid);
                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {

                    string sql1 = $@"SELECT isnull(sum(chargecost+servicecost),0) AS integral FROM histroy_order_cost A WHERE A.userid = '{openid}';
                SELECT count(*) AS card FROM lco_customer B,cfg_card C WHERE   B.customerid = C.customerid AND (B.phone='{openid}') AND B.deleteOrUser = 0 and C.OneMore=1 and C.type = 1;
                SELECT count(*) FROM cfg_card_usablephone WHERE usablePhone LIKE '%{openid}%'";

                    var otherdata = SqlMapper.QueryMultiple(conn, sql1);

                    app_user.integral = otherdata.Read<int>().FirstOrDefault();
                    app_user.card = otherdata.Read<int>().FirstOrDefault() + otherdata.Read<int>().FirstOrDefault();



                    string sql = $@"SELECT * FROM lco_wxcustomer WHERE openid = '{openid}'";
                    IEnumerable<Wxcustomer> data = SqlMapper.Query<Wxcustomer>(conn, sql);
                    if (data != null)
                    {
                        app_user.carType = data.FirstOrDefault().carType;
                        if (data.FirstOrDefault().endurance != null && data.FirstOrDefault().plate != null && data.FirstOrDefault().productionDate != null)
                        {
                            app_user.endurance = int.Parse(data.FirstOrDefault().endurance);
                            app_user.plate = data.FirstOrDefault().plate;
                            app_user.productionDate = data.FirstOrDefault().productionDate;
                        }
                    }
                }
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($@"获取用户信息失败，openid={openid},{e}");
            }

            return app_user;
        }

        /// <summary>
        /// 返回头像文件的Base64字符串
        /// </summary>
        /// <param name="profile_path">头像文件相对路径</param>
        /// <returns></returns>
        private String GetAPPUserProfile(string profile_path)
        {
            //Bitmap userProfile;
            string path = System.Web.HttpContext.Current.Server.MapPath("~/Profile/" + "profile_" + profile_path + ".png");
            if (File.Exists(path))
            {
                //userProfile = FileToBitmap(path);
                // return BitmapToString(userProfile);
                return "Profile/" + "profile_" + profile_path + ".png";
            }

            //头像文件不存在，取默认头像
            //return new UserProfile().Default;
            return "Profile/" + "profile_" + "defultimage" + ".png";
        }

        /// <summary>
        /// 根据Token获取openId
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        public string GetUserOpenId(string userToken)
        {
            if (string.IsNullOrEmpty(userToken))
            {
                return null;
            }

            var openid = GetOpenIDByUserToken(userToken);

            return openid;
        }

        public int? GetAPPUserBalance(string userToken)
        {
            if (string.IsNullOrEmpty(userToken))
            {
                return null;
            }
            APP_User user = GetAPPUser(userToken);

            return user == null ? 0 : user.Balance;
        }

        private APP_User GetAPPUserFromDB(string phoneNum)
        {
            var user = new APP_User();

            DynamicParameters param = new DynamicParameters();
            param.Add("@loginid", phoneNum);
            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var spName = "APP_GetUser";
                IEnumerable<APP_User> selectResult = connection.Query<APP_User>(spName, param, null, true, null, CommandType.StoredProcedure);
                user = selectResult.First();
            }

            return user;
        }

        private APP_User GetAPPUserFromDBbyToken(string userToken)
        {
            if (string.IsNullOrEmpty(userToken))
            {
                return null;
            }

            var openid = GetOpenIDByUserToken(userToken);
            if (string.IsNullOrEmpty(openid))
            {
                return null;
            }

            var user = new APP_User();

            DynamicParameters param = new DynamicParameters();
            param.Add("@loginid", openid);
            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var spName = "APP_GetUser";
                IEnumerable<APP_User> selectResult = connection.Query<APP_User>(spName, param, null, true, null, CommandType.StoredProcedure);
                user = selectResult.First();
            }

            return user;
        }

        public APP_User UpdateAPPUser(APP_User user, string userToken)
        {
            if (string.IsNullOrEmpty(userToken))
            {
                return null;
            }

            // userToken转openid
            var openid = GetOpenIDByUserToken(userToken);
            if (openid == "" || openid == null)
            {
                return null;
            }

            // 根据openid依次从Redis、DB中查user信息
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var app_user = redis.HashGet<APP_User>(RedisKey.APPUser, openid);
            if (app_user == null)
            {
                app_user = GetAPPUserFromDB(openid);
            }

            if (app_user == null)
            {
                return null;
            }

            // user信息存在
            APP_User newUser = new APP_User();
            DynamicParameters param = new DynamicParameters();
            param.Add("@loginid", openid);
            param.Add("@nickname", user.nickname);
            param.Add("@profile", openid);
            param.Add("@sex", user.sex);
            param.Add("@birthday", user.birthday);
            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                if (connection.State == ConnectionState.Broken || connection.State == ConnectionState.Closed)
                {
                    connection.Open();
                }
                IDbTransaction transaction = connection.BeginTransaction();
                try
                {
                    var spName = "APP_UpdateUser";
                    //IEnumerable<APP_User> selectResult = connection.Query<APP_User>(spName, param, transaction, true, null, CommandType.StoredProcedure);
                    //newUser = selectResult.First();
                    //newUser.Timestamp = user.Timestamp;
                    //newUser.DeviceID = user.DeviceID;
                    connection.Query(spName, param, transaction, true, null, CommandType.StoredProcedure);
                    app_user.nickname = user.nickname;
                    app_user.profile = openid;
                    app_user.sex = user.sex;
                    app_user.birthday = user.birthday;
                    redis.HashSet<APP_User>(RedisKey.APPUser, openid, app_user);
                    transaction.Commit();
                }
                catch (Exception ex)
                {
                    LogEvent.WebAPILoger.Error($"{user}更新信息失败", ex);
                    transaction.Rollback();
                }
            }
            app_user.profile = GetAPPUserProfile(app_user.profile);
            return app_user;
        }

        public Boolean UploadAPPUserProfile(UserProfile userProfile, string userToken, ref string Imagepath)
        {
            string accountName = System.Environment.UserName;
            string system = System.Environment.MachineName;
            accountName = "NETWORK SERVICE";

            if (string.IsNullOrEmpty(userToken))
            {
                return false;
            }

            // userToken转openid
            var openid = GetOpenIDByUserToken(userToken);
            if (openid == "" || openid == null)
            {
                return false;
            }
            string profileName = "profile_" + openid + ".png";
            string path = System.Web.HttpContext.Current.Server.MapPath("~/Profile/" + profileName);
            System.Drawing.Image image;
            if (userProfile.profileStr.Split(',').Count() > 1)
            {
                //转成图片
                image = StringToImage(userProfile.profileStr.Split(',')[1]);
            }
            else
            {
                image = StringToImage(userProfile.profileStr);

            }


            //保存头像
            //image.Dispose();
            image.Save(path, ImageFormat.Png);
            Imagepath = "/Profile/" + profileName;
            return true;
        }

        public Boolean UploadAPPUserProfile(UserProfile userProfile, string userToken, string a)
        {
            bool result = false;
            var files = HttpContext.Current.Request.Files;
            string openid = WechatUtility.GetOpenIDByUserToken(userToken);
            string profileName = "profile_" + openid + ".png";
            string path = System.Web.HttpContext.Current.Server.MapPath("~/Profile/" + profileName);
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
            try
            {
                foreach (string key in files.AllKeys)
                {
                    HttpPostedFile file = files[key];
                    if (file.ContentLength > 0)
                    {
                        if (!Directory.Exists(path))
                            Directory.CreateDirectory(path);
                        file.SaveAs(path + file.FileName);
                        userProfile.profileStr = "Profile/" + profileName;
                        result = true;
                    }
                }
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"保存用户头像照片失败openid={openid}", e);
            }
            return result;
        }
        public IEnumerable<Recharge> GetHistoryRecharge(string userToken, int minID)
        {
            IEnumerable<Recharge> recharges = new List<Recharge>();

            //处理第一次请求没有最后一条记录ID的情况
            if (minID <= 0)
            {
                minID = int.MaxValue;
            }

            var loginid = GetOpenIDByUserToken(userToken);
            if (string.IsNullOrEmpty(loginid))
            {
                return recharges;
            }

            DynamicParameters param = new DynamicParameters();
            param.Add("@loginid", loginid);
            param.Add("@minid", minID);
            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var spName = "APP_GetRecharge";
                IEnumerable<Recharge> selectResult = connection.Query<Recharge>(spName, param, null, true, null, CommandType.StoredProcedure);
                recharges = selectResult;
            }

            return recharges;
        }

        public Boolean checkToken(string token)
        {
            //token为空或无对应openId则token无效
            return (string.IsNullOrEmpty(token) || string.IsNullOrEmpty(GetOpenIDByUserToken(token))) ? false : true;
        }

        public System.Drawing.Image StringToImage(string imageStr)
        {
            byte[] imageBytes = Convert.FromBase64String(imageStr);
            //读入MemoryStream对象
            MemoryStream memoryStream = new MemoryStream(imageBytes, 0, imageBytes.Length);
            memoryStream.Write(imageBytes, 0, imageBytes.Length);
            //转成图片
            System.Drawing.Image image = System.Drawing.Image.FromStream(memoryStream);
            return image;
        }

        private Bitmap FileToBitmap(string fileName)
        {
            // 打开文件  
            FileStream fileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read);
            // 读取文件的 byte[]  
            byte[] bytes = new byte[fileStream.Length];
            fileStream.Read(bytes, 0, bytes.Length);
            fileStream.Close();
            // 把 byte[] 转换成 Stream  
            Stream stream = new MemoryStream(bytes);

            stream.Read(bytes, 0, bytes.Length);
            // 设置当前流的位置为流的开始  
            stream.Seek(0, SeekOrigin.Begin);

            MemoryStream mstream = null;
            try
            {
                mstream = new MemoryStream(bytes);
                return new Bitmap((System.Drawing.Image)new Bitmap(stream));
            }
            catch (ArgumentNullException ex)
            {
                LogEvent.WebAPILoger.Error("文件换失败+FileToBitmap", ex);
                return null;
            }
            catch (ArgumentException ex)
            {
                LogEvent.WebAPILoger.Error("文件换失败+FileToBitmap", ex);
                return null;
            }
            finally
            {
                stream.Close();
            }
        }

        private static String BitmapToString(Bitmap bitmap)
        {
            try
            {
                MemoryStream ms = new MemoryStream();
                bitmap.Save(ms, System.Drawing.Imaging.ImageFormat.Png);
                byte[] arr = new byte[ms.Length];
                ms.Position = 0;
                ms.Read(arr, 0, (int)ms.Length);
                ms.Close();
                string pic = Convert.ToBase64String(arr);

                return pic;
            }
            catch (Exception)
            {
                return null;
            }
        }

        private void UpdateAPPUserTimestamp(string phoneNum)
        {
            //RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            //var app_user = GetAPPUser(phoneNum);
            //app_user.Timestamp = DateTime.Now;
            //redis.HashSet<APP_User>(RedisKey.APPUser, phoneNum, app_user);
        }

        public List<string> NewGetChargingDevice(string userToken)
        {
            //String deviceCode = null;
            List<WXDeviceInfo> deviceInfo = NewGetDeviceInfoByUserToken(userToken);
            List<string> deviceCode = new List<string>();
            foreach (var item in deviceInfo)
            {
                if (item != null && (item.GunState == EnumGunStatus.启动中 ||
                    item.GunState == EnumGunStatus.充电中 || item.GunState == EnumGunStatus.暂停))
                {
                    deviceCode.Add(item.DeviceCode);
                }
            }
            return deviceCode;
        }

        public List<string> GetChargingDevicesByUserToken(string userToken)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var openid = redis.HashGetString(RedisKey.Token, userToken);
            return GetChargingDevices(openid);
        }

        public List<string> GetChargingDevices(string openId)
        {
            List<string> deviceCodes = null;
            if (string.IsNullOrEmpty(openId))
            {
                return deviceCodes;
            }

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            deviceCodes = redis.HashGet<List<string>>(RedisKey.ChargingDevice, openId);
            return deviceCodes;
        }

        public String GetChargingDevice(string userToken)
        {
            String deviceCode = null;
            var deviceInfo = GetDeviceInfoByUserToken(userToken);
            if (deviceInfo != null && (deviceInfo.GunState == EnumGunStatus.启动中 ||
                deviceInfo.GunState == EnumGunStatus.充电中 || deviceInfo.GunState == EnumGunStatus.暂停))
            {
                deviceCode = deviceInfo.DeviceCode;
            }
            return deviceCode;
        }

        public JObject StartCharge(string userToken, string deviceCode, string nickname)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            JObject result = new JObject();
            string openid = WechatUtility.GetOpenIDByUserToken(userToken);
            if (string.IsNullOrEmpty(openid))
            {
                throw new Exception("Token过期，请重新登陆");
            }
            string companyid = redis.HashGet<LCO_Station>(RedisKey.Station, WechatUtility.GetStationId(deviceCode)).CompanyID;
            string transationid = "VIN" + DateTime.Now.ToString("yyyyMMddHHmmssfff");
            var stationCode = deviceCode.Split('|')[0];
            //var jkCode = deviceCode.Split('|')[0];
            //var stationCode = jkCode.Length >= 10 ? jkCode.Substring(0, 10) : jkCode;
            var time_end = long.Parse(DateTime.Now.ToString("yyyyMMddHHmmss"));
            var charge = new PayLogic().CreateRedisOrderFuwushang("", openid, 999999, nickname, deviceCode, stationCode, transationid
                , transationid, time_end, EnumPaymentType.VIN, companyid);
            if (charge == true)
            {
                result.Add("code", 1);
                result.Add("data", "启动中");
            }
            else
            {
                result.Add("code", 0);
                result.Add("data", "启动失败，请稍后再试，或选择其他设备");
            }
            return result;
        }

        public JObject APPStart(string userToken, string deviceCode, int max)
        {
            JObject result = new JObject();
            // 判断用户即将启动的充电桩状态
            if (!CheckDeviceStatus(deviceCode))
            {
                result.Add("code", 0);
                result.Add("data", "设备不可用");
                return result;
            }
            List<string> chargingDevice = GetChargingDevices(userToken);
            if (chargingDevice != null && chargingDevice.Count >= max)
            {
                result.Add("code", 3);
                result.Add("data", "已启动设备达到上限");
                return result;
            }
            // 判断余额是否足够
            var user = GetAPPUser(userToken);
            int count = chargingDevice == null ? 0 : chargingDevice.Count;
            if (!CheckBalance(user.Balance, count))
            {
                result.Add("code", 2);
                result.Add("data", "余额不足");
                return result;
            }
            if (StartCharge(user.loginid, user.Balance, user.nickname, deviceCode, EnumPaymentType.app))
            {
                result.Add("code", 1);
                result.Add("data", "启动中");
            }
            else
            {
                result.Add("code", 0);
                result.Add("data", "启动失败，请稍后再试，或选择其他设备");
            }
            return result;
        }
        public JObject PlatformStartCharge(string loginID, string token, string deviceCode, int max)
        {
            JObject result = new JObject();
            // 判断平台用户身份
            if (!CheckPlatformUser(loginID, token))
            {
                result.Add("code", 0);
                result.Add("data", "身份校验未通过");
                return result;
            }
            // 判断用户即将启动的充电桩状态
            if (!CheckDeviceStatus(deviceCode))
            {
                result.Add("code", 0);
                result.Add("data", "设备不可用");
                return result;
            }
            List<string> chargingDevice = GetChargingDevices(loginID);
            if (chargingDevice != null && chargingDevice.Count >= max)
            {
                result.Add("code", 0);
                result.Add("data", "已启动设备达到上限");
                return result;
            }
            if (StartCharge(loginID, 9999999, loginID, deviceCode, EnumPaymentType.平台))
            {
                result.Add("code", 1);
                result.Add("data", "启动中");
            }
            else
            {
                result.Add("code", 0);
                result.Add("data", "启动失败，请稍后再试，或选择其他设备");
            }
            return result;
            ////修改设备状态、生成订单
            //RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            //var stationCode = deviceCode.Split('|')[0];
            ////var jkCode = deviceCode.Split('|')[0];
            ////var stationCode = jkCode.Length >= 10 ? jkCode.Substring(0, 10) : jkCode;
            //var time_end = long.Parse(DateTime.Now.ToString("yyyyMMddHHmmss"));
            //var newDeviceInfo = CreateCurrentDeviceInfo(loginID, int.MaxValue, loginID, deviceCode, stationCode,
            //    "", "", time_end, EnumPaymentType.app, "");
            //if (ChargeAndSaveOrderInfo(newDeviceInfo, redis))
            //{
            //    result.Add("code", 1);
            //    result.Add("data", "启动中");
            //}
            //else
            //{
            //    result.Add("code", 0);
            //    result.Add("data", "启动失败，请稍后再试，或选择其他设备");
            //}
            //return result;
        }

        private bool StartCharge(string openid, int total_fee, string nickName, string deviceCode, EnumPaymentType paymentType)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var stationCode = deviceCode.Split('|')[0];
            var time_end = long.Parse(DateTime.Now.ToString("yyyyMMddHHmmss"));
            var newDeviceInfo = CreateCurrentDeviceInfo(openid, total_fee, nickName, deviceCode, stationCode,
                "", "", time_end, paymentType, "");
            if (ChargeAndSaveOrderInfo(newDeviceInfo, redis))
            {
                return true;
            }
            return false;
        }

        internal bool APPStopCharge(string userToken, string deviceCode)
        {
            LogEvent.WebAPILoger.Debug($"ChargeLogic-PostStop, userToken={userToken}, deviceCode={deviceCode}");

            //从userToken查询在哪一个设备上充电
            List<WXDeviceInfo> wxDeviceInfo = NewGetDeviceInfoByUserToken(userToken);
            foreach (var item in wxDeviceInfo)
            {
                if (item.DeviceCode == deviceCode)
                {
                    // 保存平台停止的原因 app界面停止
                    RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                    redis.HashSet(RedisKey.EndResonDescription, item.OrderID, 991);
                    return StopCharge(item);
                }
            }
            return false;
        }

        internal bool PlatformStopCharge(string loginID, string token, string deviceCode)
        {
            if (!CheckPlatformUser(loginID, token))
            {
                return false;
            }
            WXDeviceInfo wxDeviceInfo = GetDeviceInfo(deviceCode);

            // 保存平台停止的原因 web界面停止
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            redis.HashSet(RedisKey.EndResonDescription, wxDeviceInfo.OrderID, 990);
            return StopCharge(wxDeviceInfo);
        }

        internal JObject APPStop(string userToken, string deviceCode)
        {
            LogEvent.WebAPILoger.Debug($"ChargeLogic-PostStop, userToken={userToken}, deviceCode={deviceCode}");

            JObject result = new JObject();
            //从userToken查询在哪一个设备上充电
            WXDeviceInfo wxDeviceInfo = GetDeviceInfo(deviceCode);
            string openId = GetUserOpenId(userToken);
            if (wxDeviceInfo != null && wxDeviceInfo.Openid == openId)
            {
                if (StopCharge(wxDeviceInfo))
                {
                    // 保存平台停止的原因 app界面停止
                    RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                    redis.HashSet(RedisKey.EndResonDescription, wxDeviceInfo.OrderID, 991);

                    List<string> chargingDevice = GetChargingDevices(userToken);
                    chargingDevice.Remove(deviceCode);
                    
                    redis.HashSet<List<string>>(RedisKey.ChargingDevice, openId, chargingDevice);
                    result.Add("code", 1);
                    result.Add("data", "指令下发成功");
                    return result;
                }
                result.Add("code", 0);
                result.Add("data", "指令下发失败");
                return result;
            }
            result.Add("code", 0);
            result.Add("data", "非本人使用设备，不能停止");
            return result;
        }

        public APP_Share GetShareInfo()
        {
            APP_Share info = new APP_Share();
            info.url = ConfigurationManager.AppSettings["shareUrl"];
            info.title = ConfigurationManager.AppSettings["shareTitle"];
            info.content = ConfigurationManager.AppSettings["shareContent"];
            info.imgUrl = ConfigurationManager.AppSettings["shareImgUrl"];
            return info;
        }

        private bool CheckPlatformUser(string loginID, string token)
        {
            if (string.IsNullOrEmpty(loginID) || string.IsNullOrEmpty(loginID))
            {
                return false;
            }
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WebAPI);
            var loginInfo = redis.HashGet<LoginInfo>(RedisKey.Login, loginID);

            if (loginInfo != null && loginInfo.Token == token && (DateTime.Now - loginInfo.Timestamp).TotalSeconds <= 3000)
            {
                return true;
            }
            return false;
        }

        public bool CheckDeviceStatus(string deviceCode)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            WXDeviceInfo wxDeviceInfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);
            if (wxDeviceInfo == null)
            {
                LogEvent.WebAPILoger.Info($"deviceCode={deviceCode}不存在");
                return false;
            }
            LogEvent.WebAPILoger.Info($"deviceCode={deviceCode}" + Enum.GetName(typeof(EnumGunStatus), wxDeviceInfo.GunState));
            if (wxDeviceInfo.GunState == EnumGunStatus.连接 ||
                wxDeviceInfo.GunState == EnumGunStatus.暂停 || wxDeviceInfo.GunState == EnumGunStatus.充满)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public bool CheckBalance(int balance, int chargingDeviceCount, int minimum, int increment, int incrementIncrement)
        {
            int need = minimum;
            for (int i = 0; i < chargingDeviceCount; i++)
            {
                need = need + increment + (i + 1) * incrementIncrement;
            }
            LogEvent.WebAPILoger.Info($"balance={balance}, need={need}");
            return balance >= need ? true : false;
        }

        public bool CheckBalance(int balance, int chargingDeviceCount)
        {
            return CheckBalance(balance, chargingDeviceCount, 1000, 1000, 0);
        }
        #endregion

        #region APP支付宝相关

        public string ALIAPPPay(WXMINAPayParameter param)
        {
            APP_User app_User = GetAPPUser(param.code);
            if (app_User == null)
            {
                app_User = GetAPPUserFromDBbyToken(param.code);
            }
            if (app_User == null)
            {
                return null;
            }
            string orderString = "";
            try
            {
                IAopClient client = new DefaultAopClient("https://openapi.alipay.com/gateway.do",
                    WxPayConfig.ALI_APP_ID, WxPayConfig.ALI_APP_PRIVATE_KEY, WxPayConfig.ALI_FORMAT, WxPayConfig.ALI_VERSION,
                    WxPayConfig.ALI_SIGN_TYPE, WxPayConfig.ALI_ALIPAY_PUBLIC_KEY, WxPayConfig.ALI_CHARSET, false);

                //实例化具体API对应的request类,类名称和接口名称对应,当前调用接口名称如：alipay.trade.app.pay
                AlipayTradeAppPayRequest request = new AlipayTradeAppPayRequest();
                //SDK已经封装掉了公共参数，这里只需要传入业务参数。以下方法为sdk的model入参方式(model和biz_content同时存在的情况下取biz_content)。
                AlipayTradeAppPayModel model = new AlipayTradeAppPayModel();
                model.Body = string.IsNullOrEmpty(ChargeLogic.WXBody) ? "充电费用" : ChargeLogic.WXBody;
                model.Subject = "充值费用";
                model.TotalAmount = (param.fee / 100.0).ToString("F2");
                model.ProductCode = "QUICK_MSECURITY_PAY";
                model.OutTradeNo = string.Format("{0}{1}", DateTime.Now.ToString("yyMMddHHmmssfff"), new Random().Next(100, 999));
                model.TimeoutExpress = "5m";
                model.PassbackParams = app_User.loginid + "|" + app_User.nickname;
                ExtendParams extendParams = new ExtendParams();
                extendParams.SysServiceProviderId = "2088821552858213";//PID
                model.ExtendParams = extendParams;
                request.SetBizModel(model);
                request.SetNotifyUrl(WxPayConfig.ALI_APP_NOTIFY_URL);

                AlipayTradeAppPayResponse response = client.SdkExecute(request);
                orderString = response.Body;
                //LogEvent.WebAPILoger.Debug($"orderString={orderString}");
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"ALIAPPPay Error param ={param.ToString()}");
                LogEvent.WebAPILoger.Error("ALIAPPPay Error=", ex);
            }

            return orderString;
        }

        public string ALIAPPPayNotify()
        {
            IDictionary<string, string> sArray = new Dictionary<string, string>();
            NameValueCollection coll = System.Web.HttpContext.Current.Request.Form;
            String[] requestItem = coll.AllKeys;
            for (var i = 0; i < requestItem.Length; i++)
            {
                sArray.Add(requestItem[i], System.Web.HttpContext.Current.Request.Form[requestItem[i]]);
                LogEvent.WebAPILoger.Info($"ALIAPPPayNotify Request.Form key={requestItem[i]},value={sArray[requestItem[i]]}");
            }

            bool RSACheckResult = AlipaySignature.RSACheckV1(sArray, WxPayConfig.ALI_ALIPAY_PUBLIC_KEY, WxPayConfig.ALI_CHARSET, WxPayConfig.ALI_SIGN_TYPE, false);

            if (RSACheckResult && sArray["trade_status"] == "TRADE_SUCCESS")
            {
                string openid = sArray["buyer_id"];
                int total_fee = (int)(decimal.Parse(sArray["total_amount"]) * 100);
                string openNickname = sArray["buyer_logon_id"];
                string phoneNumAndNickname = sArray["passback_params"];
                string[] str = phoneNumAndNickname.Split('|');
                string phoneNum = str[0];
                string loginNickname = str[1];
                string transaction_id = sArray["trade_no"];
                string out_trade_no = sArray["out_trade_no"];
                string paymentTime = sArray["gmt_payment"];
                long time_end = long.Parse(paymentTime.Replace("-", "").Replace(" ", "").Replace(":", ""));

                if (SaveRechargeRecord(phoneNum, loginNickname, openid, openNickname, total_fee, transaction_id, out_trade_no, paymentTime, EnumPaymentType.银商支付宝))
                {
                    return "success";
                }
            }
            else
            {
                LogEvent.WebAPILoger.Error($"{DateTime.Now} RSACheckV1 Result = {RSACheckResult}");
                LogEvent.WebAPILoger.Error($"{DateTime.Now} ALIAPPPayNotify sArray[\"trade_status\"] = {sArray["trade_status"]}");
            }

            return "";
        }

        private bool SaveRechargeRecord(string phoneNum, string loginNickname, string openid, string openNickname, int total_fee,
            string transaction_id, string out_trade_no, string paymentTime, EnumPaymentType paymentType)
        {
            bool result = false;

            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var isExistedSQL = "SELECT 1 FROM APP_RechargeLog WHERE loginid=@loginid AND transaction_id=@transaction_id";
                var isExistedParam = new DynamicParameters();
                isExistedParam.Add("@loginid", phoneNum);
                isExistedParam.Add("@transaction_id", transaction_id);
                IEnumerable<dynamic> isExistedResult = connection.Query(isExistedSQL, isExistedParam);
                bool isRecordExisted = (isExistedResult != null && isExistedResult.Any());

                if (isRecordExisted)
                {
                    LogEvent.WebAPILoger.Debug($"PayNotify 再次通知。phoneNum={phoneNum}，total_fee={total_fee}，paymentType={Enum.GetName(typeof(EnumPaymentType), paymentType)},transaction_id={transaction_id}");
                    result = true;
                }
                else
                {
                    // 数据库中添加充值记录
                    var insertSQL = @"INSERT INTO APP_RechargeLog (loginid, loginnickname, openid, opennickname, transaction_id, out_trade_no, payment_type, record, recordtime, invoice, invoiceid)
                                       VALUES(@loginid, @loginnickname, @openid, @opennickname, @transaction_id, @out_trade_no, @payment_type, @record, @recordtime, @invoice, @invoiceid)";
                    var insertParam = new DynamicParameters();
                    insertParam.Add("@loginid", phoneNum);
                    insertParam.Add("@loginnickname", loginNickname);
                    insertParam.Add("@openid", openid);
                    insertParam.Add("@opennickname", openNickname);
                    insertParam.Add("@transaction_id", transaction_id);
                    insertParam.Add("@out_trade_no", out_trade_no);
                    insertParam.Add("@payment_type", (int)paymentType);
                    insertParam.Add("@record", total_fee);
                    insertParam.Add("@recordtime", paymentTime);
                    //2018-05-28 充值记录新增发票信息
                    insertParam.Add("@invoice", 1);
                    insertParam.Add("@invoiceid", "000000000000000000");
                    int affectedRowNum = connection.Execute(insertSQL, insertParam);

                    if (affectedRowNum == 1)
                    {
                        RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                        var app_user = redis.HashGet<APP_User>(RedisKey.APPUser, phoneNum);
                        // Redis中不存在，则从数据库获取一次
                        if (app_user == null)
                        {
                            app_user = GetAPPUserFromDB(phoneNum);
                        }
                        else
                        {
                            // 添加此次充值费用到余额中
                            app_user.Balance += total_fee;
                        }
                        redis.HashSet(RedisKey.APPUser, phoneNum, app_user);

                        result = true;
                        LogEvent.WebAPILoger.Debug($"SaveRechargeRecord result={result}。phoneNum={phoneNum}，total_fee={total_fee}，paymentType={Enum.GetName(typeof(EnumPaymentType), paymentType)},transaction_id={transaction_id}");
                    }

                }
            }

            return result;
        }

        #endregion

        #region 开放接口相关

        public string GetDeviceStatus(string deviceCode)
        {
            //根据设备code确定此设备当前是否可用
            if (string.IsNullOrEmpty(deviceCode) || deviceCode == "undefined")
            {
                return "不存在";
            }

            GunInfo gunInfo = GetRealDataByDeviceAndSignal(deviceCode, null);
            gunInfo = gunInfo == null ? new GunInfo() : gunInfo;
            return Enum.GetName(typeof(EnumGunStatus), gunInfo.GunStatus);
        }

        public ChargeData GetRealDataByDeviceCode(string deviceCode)
        {
            WXDeviceInfo wxDeviceInfo = GetDeviceInfo(deviceCode);
            ChargeData result = null;

            if (wxDeviceInfo != null && wxDeviceInfo.OrderState != EnumWXOrderState.未开始)
            {
                result = ConvertWXDeviceInfoToChargeData(wxDeviceInfo);
            }
            else
            {
                System.Threading.Thread.Sleep(3 * 1000);
                wxDeviceInfo = GetDeviceInfo(deviceCode);
                if (wxDeviceInfo != null)
                {
                    result = ConvertWXDeviceInfoToChargeData(wxDeviceInfo);
                }
            }

            return result;
        }

        public EnumAPIStart APIStartCharge(string companyID, string appID, string deviceCode, string outTradeID)
        {
            JObject result = new JObject();
            // 判断即将启动的充电桩状态
            if (string.IsNullOrEmpty(deviceCode) || deviceCode == "undefined")
            {
                return EnumAPIStart.设备未连接;
            }
            GunInfo gunInfo = GetRealDataByDeviceAndSignal(deviceCode, null);
            EnumGunStatus status = (EnumGunStatus)gunInfo.GunStatus;
            if (status != EnumGunStatus.连接 && status != EnumGunStatus.暂停 && status != EnumGunStatus.充满)
            {
                return EnumAPIStart.设备未连接;
            }
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            RedisHelper redis5 = new RedisHelper((int)EnumUserRedisNum.Struct);
            var stationCode = deviceCode.Split('|')[0];
            var time_end = long.Parse(DateTime.Now.ToString("yyyyMMddHHmmss"));
            var deviceInfo = new WXDeviceInfo();

            var orderID = string.Format("{0}{1}", DateTime.Now.ToString("yyMMddHHmmssfff"), new Random().Next(100, 999));
            List<StrategyInfo> chargeStrategy = new List<StrategyInfo>();
            STimeInfo startTime = new STimeInfo(0, 0, 0, true);
            STimeInfo endTime = new STimeInfo(23, 59, 59, true);
            StrategyInfo strategy = new StrategyInfo();
            strategy.StartTime = startTime;
            strategy.EndTime = endTime;
            strategy.Price = 0;
            strategy.Valid = true;
            chargeStrategy.Add(strategy);
            deviceInfo.VIP = 0;
            deviceInfo.Openid = companyID + "_" + appID + "_" + outTradeID;
            deviceInfo.DeviceType = redis5.HashGet<cfg_device>(RedisKey.DeviceByDeviceCode, deviceCode).devicetype;
            deviceInfo.StationCode = stationCode;
            deviceInfo.DeviceCode = deviceCode;
            deviceInfo.NickName = companyID + "_" + appID + "_" + outTradeID;
            deviceInfo.OrderState = EnumWXOrderState.未开始;
            deviceInfo.GunState = EnumGunStatus.启动中;
            deviceInfo.Total_fee = 9999999;
            deviceInfo.RemainingFee = 9999999 * 10;
            deviceInfo.Transaction_id = "";
            deviceInfo.Out_trade_no = "";
            deviceInfo.OrderID = orderID;
            deviceInfo.ChargeStrategy = chargeStrategy;
            deviceInfo.StartTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            deviceInfo.ServiceFee = 0;
            deviceInfo.Time_end = time_end;
            deviceInfo.PaymentType = EnumPaymentType.接口;
            deviceInfo.Voltage = "";
            deviceInfo.Current = "";
            deviceInfo.Power = "";
            deviceInfo.Duration = "";
            deviceInfo.PrepayID = "";
            if (ChargeAndSaveOrderInfo(deviceInfo, redis))
            {
                return EnumAPIStart.成功;
            }
            if (StartCharge(companyID + "_" + appID + "_" + outTradeID, 9999999, companyID + "_" + appID + "_" + outTradeID, deviceCode, EnumPaymentType.接口))
            {
                return EnumAPIStart.成功;
            }
            return EnumAPIStart.其他原因失败;
        }

        public EnumAPIStop APIStopCharge(string deviceCode)
        {
            WXDeviceInfo wxDeviceInfo = GetDeviceInfo(deviceCode);
            if (wxDeviceInfo == null)
            {
                return EnumAPIStop.设备不存在;
            }
            if (wxDeviceInfo.GunState == EnumGunStatus.离线)
            {
                return EnumAPIStop.设备已离线;
            }
            if (StopCharge(wxDeviceInfo))
            {
                return EnumAPIStop.成功;
            }
            return EnumAPIStop.其他原因失败;
        }

        #endregion

        #region 发票

        public List<JObject> GetNoInvoiceHistoryOrder(string userToken, int minID)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            List<Order> orders = new List<Order>();
            List<Dictionary<string, List<Order>>> order = new List<Dictionary<string, List<Order>>>();
            Dictionary<string, List<Order>> dic = new Dictionary<string, List<Order>>();
            string openid = "";
            List <JObject> orderInfo = new List<JObject>();
            //处理第一次请求没有最后一条记录ID的情况
            if (minID <= 0)
            {
                minID = int.MaxValue;
            }
            try
            {
                openid = GetOpenIDByUserToken(userToken);
                if (string.IsNullOrEmpty(openid))
                {
                    return orderInfo;
                }
                string phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
                if (openid.Length == 11)
                {
                    phone = openid;
                }
                using (IDbConnection connection = WechatUtility.CreateDBConnection())
                {
                    var sql = $@"SELECT A.id, A.stationname,orderid,chargecost as ElectricityFee,servicecost as ServiceFee ,power,A.stationid,A.starttime,A.paymenttype,C.CompanyID
                            FROM histroy_order_cost A
                            INNER JOIN (SELECT openid AS userid FROM lco_wxcustomer WHERE phone = @phone UNION SELECT @openid) B ON A.userid=B.userid
                            LEFT JOIN lco_station C ON A.stationid=C.stationid
                            WHERE A.id < @minID AND paymenttype!=40 AND orderid  NOT IN (SELECT orderid from histroy_order_invoice)
                            UNION ALL 
                            SELECT A.id, A.stationname,orderid,chargecost as ElectricityFee,servicecost as ServiceFee ,power,A.stationid,A.starttime,A.paymenttype,C.CompanyID
                            FROM histroy_order_cost A
                            INNER JOIN (SELECT substring(openid,0,23) AS userid FROM lco_wxcustomer WHERE phone = @phone UNION SELECT @openid) B ON substring(A.userid,0,23)=B.userid
                            LEFT JOIN lco_station C ON A.stationid=C.stationid
                            WHERE A.id < @minID AND paymenttype=40 AND orderid  NOT IN (SELECT orderid from histroy_order_invoice)";
                    var parameter = new DynamicParameters();
                    parameter.Add("@phone", phone);
                    parameter.Add("@openid", openid);
                    parameter.Add("@minID", minID);
                    orders = SqlMapper.Query<Order>(connection, sql, parameter).ToList<Order>();
                }

                if (orders==null || !orders.Any())
                {
                    return orderInfo;
                }

                dic = orders.GroupBy(p => p.starttime.Month).ToDictionary(p => $"{p.Key}月份", p => p.ToList());
                foreach (var item in dic.OrderByDescending(p => p.Key))
                {
                    JObject result = new JObject();
                    result.Add("month", item.Key);
                    result.Add("data", JProperty.FromObject(item.Value));
                    orderInfo.Add(result);
                }
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"GetNoInvoiceHistoryOrder Error. userToken={userToken},openid={openid},minID={minID}",e);
            }
            return orderInfo;
        }
        public bool InsertAddressIntoDb(InvoicePurchaserAddress address)
        {
            bool result = false;
            var openid = GetOpenIDByUserToken(address.UserToken);
            if (String.IsNullOrEmpty(openid))
            {
                return false;
            }
            using (IDbConnection conn = WechatUtility.CreateDBConnection())
            {
                try
                {
                    var sql = new StringBuilder();
                    sql.Append("Insert into lco_InvoiceAddress(PostAddress,PostName,UserID,PostPhone,Title,TaxNumber,Email)");
                    sql.Append($"values('{address.PostAddress}','{address.PostName}','{openid}','{address.PostPhone}','{address.title}','{address.TaxNumber}','{address.Email}')");
                    var insertParam = new DynamicParameters();
                    insertParam.Add("@PostAddress", address.PostAddress);
                    insertParam.Add("@PostName", address.PostName);
                    insertParam.Add("@Phone", address.PostPhone);
                    insertParam.Add("@UserID", openid);
                    insertParam.Add("@Title", address.title);
                    insertParam.Add("@TaxNumber", address.TaxNumber);
                    insertParam.Add("@Email", address.Email);
                    int affectedRowNum = SqlMapper.Execute(conn, sql.ToString());
                    if (affectedRowNum == 1)
                    {
                        result = true;
                    }
                }
                catch (Exception e)
                {
                    LogEvent.WebAPILoger.Error("InsertAddressIntoDb 插入db出错", e);
                }

            }
            return result;
        }

        public IEnumerable<InvoicePurchaserAddress> GetAddressInfo(string userToken)
        {
            IEnumerable<InvoicePurchaserAddress> invoiceAddress = new List<InvoicePurchaserAddress>();
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
            var openid = GetOpenIDByUserToken(userToken);
            if (String.IsNullOrEmpty(openid))
            {
                return invoiceAddress;
            }
            string sql = "";
            using (IDbConnection conn = WechatUtility.CreateDBConnection())
            {
                try
                {
                    string phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
                    sql = $"SELECT * FROM　lco_Invoice　WHERE userID = '{openid}' or userID in (SELECT openid FROM lco_wxcustomer WHERE phone ='{phone}')";
                    invoiceAddress = SqlMapper.Query<InvoicePurchaserAddress>(conn, sql);
                }
                catch (Exception e)
                {
                    LogEvent.WebAPILoger.Error($"查询发票地址出错,sql={sql}", e);
                }

            }
            return invoiceAddress;
        }

        public bool InsertInvoiceIntoDb(Invoice invoice)
        {
            bool result = false;
            int invoiceType;
            var postArea = new StringBuilder();
            if (invoice.PostArea != null)
            {

                foreach (var area in invoice.PostArea)
                {
                    postArea.Append(area);
                }
            }


            var openid = GetOpenIDByUserToken(invoice.UserToken);

            try
            {
                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {
                    if (conn.State == ConnectionState.Broken || conn.State == ConnectionState.Closed)
                    {
                        conn.Open();
                    }
                    var addressINfo = new StringBuilder();
                    try
                    {
                        if (invoice.Title != "")
                        {
                            string sql = $@"SELECT * FROM lco_InvoiceAddress WHERE UserID='{openid}' AND PostAddress ='{invoice.PostAddress}'and email='{invoice.Email}' and title='{invoice.Title}' and postname ='{invoice.PostName}'and postphone = '{invoice.PostPhone}'  ";
                            var address = SqlMapper.Query<InvoicePurchaserAddress>(conn, sql);
                            if (address.Count() == 0)
                            {
                                addressINfo.Append("Insert into lco_InvoiceAddress(PostAddress,PostName,UserID,PostPhone,Title,TaxNumber,Email,PostArea)");
                                addressINfo.Append($"values('{invoice.PostAddress}','{invoice.PostName}','{openid}','{invoice.PostPhone}','{invoice.Title}','{invoice.TaxNumber}','{invoice.Email}','{postArea}')");
                                SqlMapper.Execute(conn, addressINfo.ToString());
                            }
                        }
                    }
                    catch (Exception e)
                    {
                        LogEvent.WebAPILoger.Error($"InsertInvoiceIntoDb SQL={addressINfo.ToString()}。", e);
                    }
                    if ((int)InvoiceType.电子 == invoice.InvoiceType)
                    {
                        invoiceType = 0;
                    }
                    else
                    {
                        invoiceType = 1;
                    }
                    using (IDbTransaction transaction = conn.BeginTransaction())
                    {
                        var Insertsql = new StringBuilder();
                        try
                        {
                            foreach (var orderInfo in invoice.order)
                            {
                                string[] orderidX = orderInfo.orderid.ToArray();
                                string a = string.Join(",", orderidX);
                                Insertsql.Clear();
                                Insertsql.Append("insert into lco_Invoice(UserID,GoodsName,TotalFee,APPID,Type,Title,TaxNumber,Email,AddressAndPhone,Remark,BankNameAndAccount");
                                Insertsql.Append($",CompanyAddress,PostAddress,PostName,PostPhone,OrderID,InvoiceType,nickname,state,AddTime,companyId) values ('{openid}','{invoice.GoodsName}',{orderInfo.TotalFee}");
                                Insertsql.Append($",'{invoice.APPID}','{invoice.Type}','{invoice.Title}','{invoice.TaxNumber}','{invoice.Email}','{invoice.AddressAndPhone}'");
                                Insertsql.Append($",'{invoice.Remark}','{invoice.BankNameAndAccount}','{invoice.CompanyAddress}','{invoice.PostAddress}'");
                                Insertsql.Append($",'{invoice.PostName}','{invoice.PostPhone}','{a}',{invoiceType},'{invoice.nickname}',{(int)InvoiceState.待处理},'{DateTime.Now}','{orderInfo.companyID}')");
                                SqlMapper.Execute(conn, Insertsql.ToString(), transaction: transaction);
                                foreach (var orderid in orderInfo.orderid)
                                {
                                    var updataSql = $"insert into  histroy_order_invoice(orderid,state)values('{orderid}',1)";
                                    SqlMapper.Execute(conn, updataSql, transaction: transaction);
                                }

                                result = true;
                            }
                            transaction.Commit();
                        }
                        catch (Exception e)
                        {
                            transaction.Rollback();
                            LogEvent.WebAPILoger.Error($"InsertInvoiceIntoDb  写入数据库失败！（更新history_order_invoice，插入Invoice）Insertsql={Insertsql.ToString()}，openid={openid}", e);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("数据库连接异常！", ex);
            }
            return result;
        }

        public IEnumerable<GetInvoice> GetInvoiceState(string userToken)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var openid = GetOpenIDByUserToken(userToken);
            string phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
            if (string.IsNullOrEmpty(openid))
            {
                throw new Exception("Token过期，请重新登陆");
            }
            IEnumerable<GetInvoice> invoice1 = new List<GetInvoice>();
            if (openid != null)
            {
                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {
                    try
                    {
                        string sql = $"select B.name,A.Type,A.Title,A.state,A.AddTime,A.TotalFee,A.CompanyID,A.FetchAddress,A.InvoiceType,A.RefuseAnswer,B.phone  from lco_invoice A,lco_manufacturer B where (A.userid='{openid}' or A.userid in (select openid from lco_wxcustomer where phone = '{phone}')) AND A.CompanyID = b.cardInfo ORDER BY A.AddTime DESC ";
                        invoice1 = SqlMapper.Query<GetInvoice>(conn, sql);
                    }
                    catch (Exception e)
                    {
                        LogEvent.WebAPILoger.Error("GetInvoice  数据库异常。", e);
                    }
                }
            }
            return invoice1;
        }

        public bool UpdataOrderState(string order)
        {
            bool result = false;
            using (IDbConnection conn = WechatUtility.CreateDBConnection())
            {
                try
                {
                    var sql = $"update history_order_invoice set state = 1 where orderid in(" + order + ")";
                    var affecrNum = SqlMapper.Execute(conn, sql);
                    if (affecrNum != 0)
                    {
                        result = true;
                    }
                }
                catch (Exception e)
                {

                    LogEvent.WebAPILoger.Error("更新数据库失败  UpdataOrderState", e);
                }

            }
            return result;
        }


        public IEnumerable<InvoicePurchaserAddress> GetInvoiceAddress(string usertoken)
        {
            IEnumerable<InvoicePurchaserAddress> invoiceAddress = new List<InvoicePurchaserAddress>();

            var openid = GetOpenIDByUserToken(usertoken);

            using (IDbConnection conn = WechatUtility.CreateDBConnection())
            {
                try
                {
                    var sql = $"SELECT * FROM　lco_InvoiceAddress　WHERE userID = '{openid}'";
                    invoiceAddress = SqlMapper.Query<InvoicePurchaserAddress>(conn, sql);
                }
                catch (Exception e)
                {
                    LogEvent.WebAPILoger.Error("GetInvoiceAddress", e);
                }

            }
            return invoiceAddress;
        }

        public bool CancleInvoice(DateTime addtime, string userToken, string companyid)
        {
            bool result = false;
            string openid = GetOpenIDByUserToken(userToken);
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
            using (IDbConnection conn = WechatUtility.CreateDBConnection())
            {
                conn.Open();
                using (IDbTransaction trans = conn.BeginTransaction())
                {
                    var command = conn.CreateCommand();
                    command.Transaction = trans;
                    string phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
                    if (openid.Length == 11)
                    {
                        phone = openid;
                    }
                    string query = $"SELECT OrderID FROM lco_Invoice WHERE ( UserID='{openid}' or UserID in (select openid from lco_wxcustomer where phone = {phone})) AND AddTime = '{addtime}' and companyid = '{companyid}'";
                    string sql = $"DELETE lco_Invoice WHERE( UserID='{openid}' or UserID in (select openid from lco_wxcustomer where phone = {phone}))AND AddTime='{addtime}' and companyid='{companyid}'";

                    // string orders = SqlMapper.Query(conn, query).ToString();
                    command.CommandText = query;
                    string orders = command.ExecuteScalar().ToString();
                    string[] order = orders.Split(',');
                    for (int i = 0; i <= order.Length - 1; i++)
                    {
                        string deletesql = $"DELETE histroy_order_invoice WHERE orderid = '{order[i]}'";
                        command.CommandText = deletesql;
                        command.ExecuteNonQuery();
                    }
                    command.CommandText = sql;
                    command.ExecuteNonQuery();
                    trans.Commit();
                    result = true;
                }

            }

            return result;


        }

        #endregion

        #region 支付方式选择、所属平台
        public EnumPayChannel GetPaymentMethod(string stationid, ref EnumPayChannel paychannel, ref EnumAppName app)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);

            try
            {
                //哪一个小程序
                List<string> stationofXM = redis.HashGet<List<string>>(RedisKey.StationOfApp, "xiaoma");
                if (stationofXM != null && stationofXM.Contains(stationid))
                {
                    app = EnumAppName.小马;
                }
                else
                {
                    app = EnumAppName.领充;
                }

                //哪一种支付方式
                List<string> stationOfPay = redis.HashGet<List<string>>(RedisKey.StationOfApp, "UMSPay");
                if (stationOfPay != null && stationOfPay.Contains(stationid))
                {
                    paychannel = EnumPayChannel.银联商务;
                }
                else
                {
                    paychannel = EnumPayChannel.微信;
                }
                
                //LCO_Station station = redis.HashGet<LCO_Station>(RedisKey.Station, stationid);
                //switch (station.payChannel)
                //{
                //    case EnumPayChannel.威富通:
                //        payway = EnumPayChannel.威富通;
                //        break;
                //    case EnumPayChannel.微信:
                //        payway = EnumPayChannel.微信;
                //        List<string> stationOfXM = redis.HashGet<List<string>>(RedisKey.StationOfApp, "xiaoma");

                //        if (stationOfXM != null && stationOfXM.Contains(stationid))
                //        {
                //            app = EnumAppName.小马;
                //        }
                //        break;
                //    case EnumPayChannel.银联商务:
                //        payway = EnumPayChannel.银联商务;
                //        break;
                //    default:
                //        payway = EnumPayChannel.微信;
                //        break;
                //}
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error(e);
            }
            finally
            {
                //payway = EnumPayChannel.微信;
                //app = EnumAppName.领充;
            }

            return paychannel;
        }
        #endregion

    }
}